/* convert a structure factor file from one format to another */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include "sf_convert.h"
#include "util.h"
#include "TableFile.h"
#include "CifFile.h"
#include "CifParserBase.h"


/*The hard coded variables (MTZ2CIF, URL_USERS_DATA, USERS_DATA,EXECUTE)
are useless for the Unix public version
*/

/*
// when used in Linux34 for root /WWW-pdb-extract/ (CGI related)
const char *MTZ2CIF= "/WWW-pdb-extract/prog-vari/sf-converter/mtz2cif-0.6/mtz2cif";
const char *CCP4= "/WWW-pdb-extract/ccp4-6.0.1";
const char *URL_USERS_DATA="http://pdb-extract.rcsb.org/auto-check/users_data";
const char *USERS_DATA="/WWW-pdb-extract/docs/auto-check/users_data";
const char *EXECUTE="/WWW-pdb-extract/prog-vari/bin/sf_convert";
const char *PATHTMP="/scr/";
*/

// when used in ws-29 
const char *MTZ2CIF= "/home/hyang/prog-vari/bin/mtz2cif";
const char *CCP4= "$CCP4";  
const char *URL_USERS_DATA="http://pdb-ws-29.rutgers.edu/auto-check/users_data";
const char *USERS_DATA="/var/www/html/auto-check/users_data";
const char *EXECUTE="/home/hyang/prog-vari/bin/sf_convert";
const char *PATHTMP="/tmp/";

//===============================================================
const char *CGI_PATH= "/cgi-bin/auto-check";  //common for Ws-29 & Linux34
//===============================================================

// externals 
int  TEST=0, SINGLE=0, FORMAT=0, DATA_SET=1;
char SYMM[20];
float CELL[6];

void multi_file2mmcif(int argc,char **argv,char *outfile,char *format_inp,
                      char *pdb_id, char *type);


int main(int argc, char **argv)
{
    char inpfile[512], outfile[512],outfile_tmp[512], pdbfile[512];
    char format_inp[20], format_out[20], cns_items[512];
    char cgi_path[512],  pdb_id_inp[256], I_F_type[10];
    int i, j,mtz_cgi=0, cns_cgi=0, cns_man=0, mf=1;
    float flag=0;
    

    strcpy(pdb_id_inp, "rXXXXsf");
    strcpy(cns_items, "");
    strcpy(outfile, "");
    strcpy(pdbfile, "");
    
    check_error_message(argc, argv);
    
    for(i=0; i<argc-1; i++){
        if(!strcmp_case(argv[i], "-i")){
            strcpy(format_inp, argv[i+1]);
            
        }else if(!strcmp_case(argv[i], "-o")){
            strcpy(format_out, argv[i+1]);
            
        }else if(!strcmp_case(argv[i], "-sf")){
            strcpy(inpfile, argv[i+1]);
            file_handle(argv[i+1], inpfile);
            mf=0;
            for(j=i+1; j<argc;j++){ //multifiles to mmcif
                if(argv[j][0]=='-'){
                    i=j-1;
                    break;
                }
                mf++;
            }
            
        }else if(!strcmp_case(argv[i], "-out")){
            strcpy(outfile, argv[i+1]);

            
        }else if(!strcmp_case(argv[i], "-pdb")){
            if(argv[i+1][0] != '-'){
                strcpy(pdbfile, argv[i+1]);
                get_cell_pdb(pdbfile, pdb_id_inp);
            }
            
        }else if(!strcmp_case(argv[i], "-man")){
            get_cell_man(argv, argc, i, &j);
            i=j;
            
            
        }else if(!strcmp_case(argv[i], "-flag")){
            if(argv[i+1][0] != '-'){
                sscanf(argv[i+1], "%f", &flag);
                if(flag>1)
                    TEST=int(100/flag + 0.5);
                else{
                    TEST=int(100/8 + 0.5);  // 8%
                    printf("The flag is < 1%%, 8%% has been assigned\n");
                }
           }
            
        }else if(!strcmp_case(argv[i], "-sf_type")){ //I or F
            if(argv[i+1][0] != '-')strcpy(I_F_type, argv[i+1]);
            
        }else if(!strcmp_case(argv[i], "-path")){
            strcpy(cgi_path, argv[i+1]);
            
        }else if(!strcmp_case(argv[i], "-pdb_id")){
            strcpy(pdb_id_inp, argv[i+1]);
            
            
        }else if(!strcmp_case(argv[i], "-cns_man")){
            for(j=i+1; j<argc;j++){
                if(argv[j][0]=='-'){
                    i=j-1;
                    break;
                }
                strcat(cns_items, argv[j]);
            }
            cns_man =1;
            
        }else if(!strcmp_case(argv[i], "-mtz_man_html")){ //only for CGI
            mtz_cgi =1;
            if(atoi(argv[i+1])>=1) DATA_SET=atoi(argv[i+1]);
            
            
        }else if(!strcmp_case(argv[i], "-cns_man_html")){ //only for CGI
            cns_cgi =1;
            if(atoi(argv[i+1])>=1) DATA_SET=atoi(argv[i+1]);
            
        }else if(!strcmp_case(argv[i], "-valid")){ //only for CGI
            validate_sf_format(argv[i+1]);
        }else if(!strcmp_case(argv[i], "-remtz")){ //only for CGI
            reformat_mtz_output(argv[i+1]);
            exit(0);
            
        }else if(!strcmp_case(argv[i], "-reformat")){ 
            cifparse(argv[i+1],argv[i+1], 80 );
        
        }
    }
    
    if(strlen(outfile)<=0) {sprintf(outfile, "%s.%s",inpfile, format_out);}

    if(mf>1){//multifile
        if(!strstr_case(format_inp, "mmcif") && strstr_case(format_out, "mmcif")){
            printf("Multifile conversion. File number=%d\n", mf);
            printf("\nInput File Name = %s : (format=%s)\n",inpfile, format_inp);
            multi_file2mmcif(argc,argv,outfile,format_inp, pdb_id_inp, I_F_type);
            exit(0);
        }
    }
    
    for(i=0; i<argc; i++){
        if(!strcmp_case(argv[i], "-single")){
            SINGLE=1;
        }else if(!strcmp_case(argv[i], "-format")){
            FORMAT=1;
        }
    }
    
    if(FORMAT>0){ 
        guess_sf_format(inpfile);
        exit(0);
    }
    
    if(mtz_cgi>0){ //only for CGI
        get_mtz_infor(inpfile, cgi_path, pdb_id_inp);
        exit(0);
    }
    
    if(cns_cgi>0){ //only for CGI
        get_cns_html(inpfile, cgi_path, pdb_id_inp);
        printf("The HTML file is %scns_man_convert.html\n",PATHTMP);
        exit(0);
    }
    
    
    if(cns_man>0){
//manual conversion for CNS, the input must be seperated by ','
// e.g.  -cns_man IOBS=,SIGI=,FOBS=,SIGMA=,T=               
        printf("Manual conversion from CNS to mmCIF format ...\n");
        
        get_cns_man(inpfile, outfile, cns_items, pdb_id_inp);
        exit(0);
    }
    

//   converting start      
    printf("\nInput File Name = %s : (format=%s)\n",inpfile, format_inp);
    

    if(!strcmp_case(format_inp, "mmcif") && !strcmp_case(format_out, "mmcif")){
//mmcif to mmcif conversion:        
        cif2cif_sf_all(inpfile, outfile,pdb_id_inp);
         
    }else if(!strcmp_case(format_inp, "mmcif") && strcmp_case(format_out, "mmcif")){
//mmcif to non-mmcif conversion:        
        cif_2_various(inpfile, pdbfile, outfile, format_out);
        
    }else if(strcmp_case(format_inp, "mmcif") && !strcmp_case(format_out, "mmcif")){
//non-mmcif to mmcif conversion:        
        various_2_cif(inpfile, outfile,format_inp, pdb_id_inp, I_F_type);
        
    }else {
//non-mmcif to non-mmcif conversion:     (bridged by mmcif)    
        sprintf(outfile_tmp, "%s_tmp.cif", outfile); 
        various_2_cif(inpfile, outfile_tmp, format_inp, pdb_id_inp, I_F_type);
        cif_2_various(outfile_tmp, pdbfile, outfile, format_out);
    }
    
    if(strlen(pdbfile)>0 && !strcmp_case(format_out, "mmcif") ){
        make_complete_SF(pdbfile, outfile);
    }
    
    printf("\nOutput File Name = %s : (%s format)\n\n", outfile, format_out);
    delete_file("CifParser.log");
}

void multi_file2mmcif(int argc,char **argv,char *outfile,char *format_inp,
                      char *pdb_id, char *type)
//combine all together.
{
    char out_tmp[2560],outfile_tmp[256], command[256];
    int i, j;

    for(i=1; i<argc; i++){
        if(!strcmp_case(argv[i], "-sf")){
            for(j=i+1; j<argc; j++){
                if(argv[j][0]=='-'){break;}
                printf("\nProcessing file = %s\n", argv[j]);
                sprintf(outfile_tmp, "%s_%d.tmp", outfile,j);
                
                various_2_cif(argv[j], outfile_tmp, format_inp, pdb_id, type);
                strcat(out_tmp,outfile_tmp);
                strcat(out_tmp," ");
            }
            printf("\nThe output file name = %s\n", outfile);
            
        }
    }
    sprintf(command, "/bin/csh -c \" /bin/cat  %s  > %s\" ", out_tmp,outfile);
    system(command);
    delete_file(outfile_tmp);
    
}

void make_complete_SF(char *pdbfile, char *outfile)
// add categories if exist
{
    char **line1=NULL, **line2=NULL, date[40],pdbid[10], symm[20];
    int i, nline1, nline2, n1=0,n2=0,n3=0,n4=0,n5=0, nn;
    float cell[6], wave;
    FILE *fo;

    
    line1=get_lines_from_file(pdbfile, &nline1);
    line2=get_lines_from_file(outfile, &nline2);
    
    fo=fopen(outfile, "w");

    for(i=0; i<6; i++)cell[i]=0;
    strcpy(date, "?");
    strcpy(symm, "?");
    strcpy(pdbid, "XXXX");
    for(i=0; i<nline1; i++){ //PDB infor.
        if(!strncmp(line1[i], "ATOM  ", 6) || !strncmp(line1[i], "HETATM", 6)) break;
        if (!strncmp(line1[i],"HEADER ", 7)){
//            sscanf(line1[i]+49, "%s", date);
            sscanf(line1[i]+61, "%s",  pdbid);
            
        }else if(!strncmp(line1[i],"CRYST1 ", 7)){
            sscanf(line1[i]+7, "%f%f %f%f %f%f ",
                   &cell[0], &cell[1], &cell[2], &cell[3], &cell[4], &cell[5]);
            get_string(line1[i],54,64,symm );
            rid_of_front_end_space(symm);
            
        }else if(strstr(line1[i],"REMARK 200  WAVELENGTH OR RANGE        (A) :")){
            sscanf(strchr(line1[i], ':')+1,"%f", &wave);
            
        }
    }
    if(line1 && nline1)free_memory_2d(line1, nline1);
    
    
    for(i=0; i<nline2; i++){ //SF info
        rid_of_front_space(line2[i]);
        if(!strncmp(line2[i], "_audit.", 7)){
            n1=1;
        }else if (!strncmp(line2[i], "_cell.length_a", 14)){
            n2=1;
        }else if (!strncmp(line2[i], "_symmetry.", 9)){
            n3=1;
        }else if (!strncmp(line2[i], "_diffrn_radiation.", 18)){
            n4=1;
        }else if (strstr(line2[i], "_diffrn_radiation_wavelength.wavelength")){
            n5=1;
        }else if(i>100 && !strncmp(line2[i],"data_",5)) break; 
    }

//    printf("n1, ...n4 = %d %d %d %d %d %d :: %s\n", n1, n2,n3,n4,nline1, nline2, outfile );
    
    nn=0;
    for(i=0; i<nline2; i++){ //write SF info
        fprintf(fo, "%s", line2[i]);
        if(!strncmp(line2[i],"data_",5)) {
            nn++;
            if(nn==1 && n1==0) {
                fprintf(fo,"\n#\n");
                fprintf(fo,"_audit.revision_id      1_0\n");
                fprintf(fo,"_audit.creation_date  '%s'\n", date);
                fprintf(fo,"_audit.update_record    'Initial release'\n");  
            }
            if(nn==1 && n2==0) {
                fprintf(fo, "\n#\n_cell.entry_id  %s\n", pdbid);
                fprintf(fo, "_cell.length_a  %.3f\n", cell[0] );
                fprintf(fo, "_cell.length_b  %.3f\n", cell[1]);
                fprintf(fo, "_cell.length_c  %.3f\n", cell[2]);
                fprintf(fo, "_cell.angle_alpha  %.3f\n", cell[3]);
                fprintf(fo, "_cell.angle_beta  %.3f\n", cell[4]);
                fprintf(fo, "_cell.angle_gamma  %.3f\n", cell[5]);
            }
            if(nn==1 && n3==0) {
                fprintf(fo, "\n# \n_symmetry.entry_id %s\n", pdbid);
                fprintf(fo, "_symmetry.space_group_name_H-M '%s'\n", symm); 
            }
            
            if(nn==1 && n4==0 && n5==0) {
                fprintf(fo, "\n# \n_diffrn_radiation.id   1\n");
                if(wave>0){
                    fprintf(fo, "_diffrn_radiation.pdbx_wavelength    %.5f\n", wave);
                }
                
            }
            
        }
        
    }
    if(line2 && nline2)free_memory_2d(line2, nline2);
    fclose(fo);
}

void usage()
{
    printf("\n");
    printf("Usage: sf_convert  -i input_format -o output_format -sf data_file\n");
    printf("-i     input format: \n");
    printf("       mmCIF, CIF, MTZ, CNS, Xplor, HKL2000, Scalepack, Dtrek, TNT, \n");
    printf("       SHELX, SAINT, EPMR, XSCALE, XPREP, XTALVIEW, X-GEN, XENGEN, \n");
    printf("       MULTAN, MAIN, OTHER.\n");
    printf("-o     output format:\n");
    printf("       mmCIF, MTZ, CNS, TNT, SHELX, EPMR, XTALVIEW, HKL2000, Dtrek,\n");
    printf("       XSCALE, MULTAN, MAIN, OTHER.\n");
    printf("-sf    input structure factor file name.\n");
    printf("\nOptional options:\n");
    printf("-out   followed by output file name (If not given, default by program)\n");
    printf("-pdb   followed by PDB file (must contain symmetry and cell parameters)\n");
    printf("-man   manually typing in cells & symmetry (a,b,c,alpha,beta,gamma,p21)\n");
    printf("-flag  followed by number (e.g. 8 percent of reflection for free R)\n");
    printf("-sf_type  followed by I or F. \n");
    printf("-format  guess the format of the SF file \n");
    
    printf("\nNote:  \n");
    printf("    -pdb or -man is ONLY for conversion to MTZ file (CCP4 must be installed).\n");
    printf("    If -pdb is used, PDB file must have symmetry and cells\n");
    printf("    If -man is used, seperate each item by a comma ',' without space.\n");
    printf("    CIF and mmCIF are for small molecule and macro-molecule format\n");
    
    printf("\n");
    printf("    If input format is OTHER, you should provide a ASCII file \n");
    printf("    with H, K, L, F, SigmaF seperated by space.\n");
    
    printf("\nExample of CNS to mmCIF conversion:  \n");
    printf("    sf_convert  -i cns -o mmcif -sf sf_file_name\n");
    printf("\nExample of Scalepack to MTZ conversion:  \n");
    printf("    sf_convert  -i Scalepack -o MTZ  -sf sf_file \n");
    printf("    or  \n");
    printf("    sf_convert  -i Scalepack -o MTZ  -sf sf_file -man  a,b,c,alpha,beta,gamma,P21\n\n");
    printf("Default data type: F for EPMR, TNT, OTHER, MULTAN, MAIN,XTALVIEW;\n");
    printf("                   I for SHELX,HKL2000,DTREK,XSCALE,XPREP,SAINT;\n\n");
    exit(0);
}

void guess_sf_format(char *inpfile)
// guess various format
{
    char **line,  outfile[256], str[512], tmp[512];
    char s1[80],s2[80],s3[80],s4[80],s5[80],s6[80],s7[80];
    int i, nline, len, n=0, n1, n2,n3,n4, k1,k2,k3, m1,m2, mm1=0;
    float a;
    
    FILE *fout=NULL;

    strcpy(outfile, "sf_format_guess.text");
    fout=fopen(outfile, "w");
    
    line=get_lines_from_file(inpfile, &nline);

    printf("Number of lines =%d\n", nline);
    if(nline<200){
        printf("Error! Input file is wrong!\n");
        fprintf(fout,"Error! Input file is wrong!\n");
        fclose(fout);
        exit(0);
    }
    
    n=0; //guess MTZ
    for(i=0; i<nline; i++){
        if(i>20) break;
        if(line[0][0] =='M' && line[0][1] =='T' && line[0][2] =='Z' && line[0][3] ==' ' ) {
            write_format(fout,"MTZ", 10);
        }else
            break;
    }
    
    
    n1=0; //guess mmCIF
    n2=0; //guess CIF
    n3=0; //guess CNS (MAIN)
    n4=0; //guess TNT
    k1=0;k2=0;k3=0;
    m1=0;m2=0;
    
    for(i=0; i<nline; i++){
        if(i>1000) break;
        strcpy(tmp,line[i]);
        
        sscanf(tmp, "%s", str);
            
        if(!strncmp_case(str, "_refln.", 7)){
             if(!strcmp_case(str, "_refln.F_meas_au")) k1=1;
             if(!strcmp_case(str, "_refln.intensity_meas" ))k2=1;
             if(!strcmp_case(str, "_refln.pdbx_F_plus" ) ||
                 !strcmp_case(str, "_refln.pdbx_I_plus" ))k3=1;
            
            n1++;
        }else if(!strncmp_case(str, "_refln_", 7)){
            if(!strcmp_case(str, "_refln_F_meas" ))k1=1;
            if(!strcmp_case(str, "_refln_F_squared_meas" ))k2=1;
            
            n2++;
        }else if(!strncmp_case(str, "INDE", 4)){
            if(strstr_case(line[i], "FOBS=" ) ||
                 strstr_case(line[i], "FO=" )||
               strstr_case(line[i], "F=" ))m1++;
            
            if(strstr_case(line[i], "IOBS=" ) ||
                 strstr_case(line[i], "IO=" )||
               strstr_case(line[i], "I=" ))m2++;

            if(strstr_case(line[i], "FOBS " ) && strstr_case(line[i], "SIGMA " ) )mm1++;
            
            n3++;
        }else if(!strncmp_case(str, "HKL", 3)){
            sscanf(line[i], "%*s%*s%*s%*s%f", &a);
            if(a<0)m2++;
            n4++;
        }else{
            if(strstr_case(line[i], "FOBS=" ) ||
                 strstr_case(line[i], "FO=" )||
               strstr_case(line[i], "F=" ))m1++;
            
            if(strstr_case(line[i], "IOBS=" ) ||
                 strstr_case(line[i], "IO=" )||
               strstr_case(line[i], "I=" ))m2++;
        }
        
    }
//    printf("m1=m2= %d %d %d %d %d \n", m1, m2, k1, k2,k3);
    
    if(n1>=4){
        if(k1==1 && k2==1){
            write_format(fout,"MMCIF", 2); //I&F
        }else if(k1==1 && k2!=1){
            write_format(fout,"MMCIF", 1); //F
        }else if(k1!=1 && k2==1){
            write_format(fout,"MMCIF", 0); //I
        }else if(k3==1){
            write_format(fout,"MMCIF", 3); //ano
        }else
            write_format(fout,"MMCIF", -1); //nothing
        
    }else if(n2>=4){
        if(k1==1 && k2==1){
            write_format(fout,"CIF", 2); //I&F
        }else if(k1==1 && k2!=1){
            write_format(fout,"CIF", 1); //F
        }else if(k1!=1 && k2==1){
            write_format(fout,"CIF", 0); //I
        }else
            write_format(fout,"CIF", -1); //nothing
        
    }else if(n3>=100){
        if(m1>100 && m2>100){
            write_format(fout,"CNS", 2); //I&F
        }else if(m1>100 &&  m2<10){
            write_format(fout,"CNS", 1); //F
        }else if(m2>100 &&  m1<10){
            write_format(fout,"CNS", 0); //I
        }else if(mm1>100 ){
            write_format(fout,"MAIN", 1); //I
        }else
            write_format(fout,"CNS", -1); //nothing
        
    }else if(n4>=100){
        if(m2>0)
            write_format(fout,"TNT", 0);
        else
            write_format(fout,"TNT", -2);
    }
    

    n1=0; //guess XSCALE based on head
    n2=0; //guess d*TREK based on head
    n3=0; //guess HKL/SCALEPACK based on head
    for(i=0; i<nline; i++){
        if(i>50) break;
        n=sscanf(line[i], "%s%s%s%s%s%s%s",s1,s2,s3,s4,s5,s6,s7);
            
        if(!strncmp_case(s1, "!SPACE_GROUP_NUMBER=", 20) ||
           !strncmp_case(s1, "!UNIT_CELL_CONSTANTS=", 21) ||
           !strncmp_case(s1, "!ITEM_H=",8) ||
           !strncmp_case(s1, "!ITEM_K=",8) ||
           !strncmp_case(s1, "!ITEM_L=",8) ){
            n1++;
        }else if(!strncmp_case(s1, "CRYSTAL_MOSAICITY=", 18)||
                 !strncmp_case(s1, "CRYSTAL_SPACEGROUP=",19) ||
                 !strncmp_case(s1, "CRYSTAL_UNIT_CELL=",18) ||
                 !strncmp_case(s1, "nH",2) ||
                 !strncmp_case(s1, "nK",2) ||
                 !strncmp_case(s1, "nL",2) 
                 ){
            
            n2++;
        }else if(i==0 && !strcmp_case(s1, "1")){
            n3++;
        }else if(i==1 && !strcmp_case(s1, "-985") || !strcmp_case(s1, "-987")){
            n3++;
        }else if(i==2 && n==7){
            n3++;
        }  
    }
    if(n1>=5){
        write_format(fout,"XSCALE", 0);
    }else if(n2>=6){
        write_format(fout,"DTREK", 0);
    }else if(n3==3){
        write_format(fout,"HKL", 0);
    }

    
    k3=0;  //guess saint
    k1=0; k2=0;  //guess SHELX
    n1=0; n2=0; //guess OTHER
    m1=0;m2=0;
    for(i=0; i<nline; i++){
        rid_of_end_space(line[i]);
        n=sscanf(line[i], "%s%s%s%s%s%s",s1,s2,s3,s4,s5,s6);
        if(atoi(s1)<-500 || atoi(s1)>500 ||
           atoi(s2)<-500 || atoi(s2)>500 ||
           atoi(s3)<-500 || atoi(s3)>500 ){
            write_format(fout,"", -1);
        }
        
        len=strlen(line[i]);
        
        if(len==28){
            strncpy(s1, line[i]+12, 8);
            s1[8] = '\0';
            if(atof(s1)<0) m2++; //I
            
            k1++;
        }else if(len==32){
            strncpy(s1, line[i]+12, 8);
            s1[8] = '\0';
            if(atof(s1)<0) m2++; //I
            
            strncpy(s2, line[i]+28, 4);
            s2[4] = '\0';
            if(atoi(s2)==1 || atoi(s2)==-1){ 
                k2++;
            }
        }else if(len>150){
            k3++;
        }
        if(n==5){
            if(atof(s4)<0) m1++; //I
            n1++;
        }else if(n==6){
            if(atof(s4)<0) m1++; //I
            n2++;
        }      
        
    }
    
    
    if(k3==nline){
        write_format(fout,"SAINT", 0);
    }else if(k1==nline || k2==nline){
        if(m2>0)
            write_format(fout,"SHELX", 0);
        else
            write_format(fout,"SHELX", -2);
    }else if(n1==nline || n2==nline){
        if(m2>0)
            write_format(fout,"OTHER", 0);
        else
            write_format(fout,"OTHER", -2);
        
    }else{
        write_format(fout,"", -1);
    }
    
}

void  write_format(FILE *fout, const char *format,int key)
{
    int len=0;

    len=strlen(format);
    if(len<1){
        printf("Sorry! format can not be predicted.\n");
        fprintf(fout,"Sorry! format can not be predicted.\n");
    }else{
        printf("Input file FOMRAT=%s\n", format);
        if(key==-1) printf("Data type can not be predicted. Please use the validation programs (SFCHECK, REFMAC ..) to determine the data type!\n");
        if(key==-2) printf("Data type may be Amplitude(F), but you need validation programs (SFCHECK, REFMAC ..) to confirm!\n");
        
        if(key==0) printf("Data type is Intensity (I).\n");
        if(key==1) printf("Data type is Amplitude (F).\n");
        if(key==2) printf("Data type are both Amplitude (F) and Intensity (I).\n");
        if(key==3) printf("Data type are anomolous signal.\n");
        if(key==10) printf("Data type for MTZ file is auto_converted.\n");
        
        fprintf(fout,"FOMRAT=%s\n", format);
        if(key==-1) fprintf(fout,"Data type can not be predicted. Please use the validation programs (SFCHECK, REFMAC ..) to determine the data type!\n");
        if(key==-2) fprintf(fout,"Data type may be Amplitude(F), but you need validation programs (SFCHECK, REFMAC ..) to confirm!\n");
        if(key==0) fprintf(fout,"Data type is Intensity (I).\n");
        if(key==1) fprintf(fout,"Data type is Amplitude (F).\n");
        if(key==2) fprintf(fout,"Data type are both Amplitude (F) and Intensity (I).\n");
        if(key==3) fprintf(fout,"Data type are anomolous signal.\n");
        if(key==10) fprintf(fout,"Data type for MTZ file is auto_converted.\n");
    }
    
    fclose(fout);
    exit(0);
}


void get_cell_pdb(char *pdbfile, char *pdb_id_inp)
{
    char str[100], symmetry_in[20], tmp[10];    
    FILE *fpdb;
    
    if((fpdb=fopen(pdbfile, "r"))==NULL){
        open_file_error(pdbfile, "get_cell_pdb");
    }
    
    while(fgets(str, sizeof str, fpdb)!=NULL){
        if (!strncmp(str,"HEADER ", 7)){
            get_string(str,61,66, tmp);
            rid_of_space(tmp);
            if(strlen(tmp)==4){
                lowerstr(tmp);
                strcpy(pdb_id_inp, make_string("r", tmp, "sf"));
            }
            
        }else if (!strncmp(str,"CRYST1 ", 7)){
            sscanf(str+7, "%f%f %f%f %f%f ",
                   &CELL[0], &CELL[1], &CELL[2], &CELL[3], &CELL[4], &CELL[5]);
            
            get_string(str,54,64, symmetry_in);
            space_group_convertor(symmetry_in, SYMM);
//            printf("symmetry_in= %s; standard=%s\n",symmetry_in  , SYMM);
            break;
        }
    }
    fclose(fpdb);

}

void get_cell_man(char **argv, int argc, int in, int *out)
{
    char tmp[256], **line;
    int i, nstr;
    
    if(argv[in+1][0] =='-'){
        printf("Wrong input! It should be '-man  a,b,c,alpha,beta,gamma,SYMM' (no space)\n");
        *out=in+1;
        return;
    }
    
    strcpy(tmp,argv[in+1]);
    
    for(i=in+2; i<argc; i++){
        if(argv[i][0]=='-') break;strcat(tmp," , ");
        strcat(tmp,argv[i]);
    }
    rid_of_space(tmp);
    
    *out=i-1;
    
    line = string_token(tmp,",", &nstr);
    if(nstr!=7){
        printf("Wrong input! Number of input (%d) < 7\n", nstr);
        printf("It should be '-man  a,b,c,alpha,beta,gamma,P21'\n");
        return;
    }
    CELL[0]=atof(line[0]);
    CELL[1]=atof(line[1]);
    CELL[2]=atof(line[2]);
    CELL[3]=atof(line[3]);
    CELL[4]=atof(line[4]);
    CELL[5]=atof(line[5]);
    
    strcpy(SYMM, line[6]);
//    printf("cell: a= %.2f c= %.2f alpha= %.2f : symmetry_in= %s\n", CELL[0], CELL[2],CELL[3], SYMM);

}


void get_cns_man(char *inpfile, char *outfile, char *cns_items, char *pdb_id)
//manual conversion  (to cif) of CNS format by typing in cns_items
{
    char str[1000], tmp[80], **token , space[10];
    int i, n,  nref=0, ntoken=0, H, K, L;
    
    FILE *finp, *ftmp, *fout;
    
    token = string_token(cns_items,",", &ntoken);
    if(ntoken<1) {
        printf("Error! There are not enough data members after -cns_man\n");
        return;
    }
    
    if((fout=fopen(outfile,"w"))==NULL){
        open_file_error(outfile,"get_cns_man");
    }

    
    fprintf(fout,"data_%s\n",pdb_id);
    
    fprintf(fout,"#\n%s\n%s\n%s\n#\n",
            "_audit.revision_id      1_0 ",
            "_audit.creation_date  ?",
            "_audit.update_record    'Initial release' "
            );
    
    fprintf(fout, "#\n");

    fprintf(fout, "loop_\n");
    fprintf(fout, "_refln.crystal_id \n");
    fprintf(fout, "_refln.wavelength_id \n");
    fprintf(fout, "_refln.scale_group_code\n");
    fprintf(fout, "_refln.index_h\n");
    fprintf(fout, "_refln.index_k\n");
    fprintf(fout, "_refln.index_l\n");
    
    strcpy(space," ");
    write_cns_data_items(fout, space, token, ntoken, 0);//only token names
    
    
    if((finp=fopen(inpfile,"r"))==NULL){
        open_file_error(inpfile,"get_cns_man");
    }

    
    ftmp = tmpfile();
    
    while (fgets(str, sizeof str, finp) != NULL) {
        sscanf(str,"%s",tmp);
            
        if(strstr_case(tmp, "INDE")){
            fprintf(ftmp, "\n");
            nref++;
        }
        n=strlen(str);
        for(i=0; i<n; i++){
            if(isspace(str[i])) str[i]=' '; 
            fprintf(ftmp, "%c", str[i]);
        }
    }
    
    if(nref < 10  ){
        printf("There are too few reflections in the file (%s).\n", inpfile);
        return;
    }
    
    rewind(ftmp);

    while (fgets(str, sizeof str, ftmp) != NULL) {
        rid_of_front_space(str);
        
        if(strstr_case(str, "INDE")){
            sscanf(str,"%*s %d %d %d", &H, &K, &L);
            
            fprintf(fout, "1 1 1 %4d %4d %4d ",H, K, L);
            
            write_cns_data_items(fout, str, token, ntoken, 1);
            
            fprintf(fout, "\n");
            
        }
    }
    fprintf(fout, "#END OF REFLECTIONS\n");
    fclose(ftmp);
    fclose(finp);
    fclose(fout);
        
    printf("The output file = %s\n",outfile);
    

}

void write_cns_data_items(FILE *fout, char *str, char **token, int ntoken, int key)
// write CNS items
{
    char item[20];
    int i, n;
    float value;

   
    for(i=0; i<ntoken; i++){
        strcpy(item, strstr(token[i], "=")+1);
        if(key==0){
            printf("data items: %d %s\n",i, token[i]);
        }
        strcat(item,"=");
        
        if(!strncmp_case(token[i], "FOBS", 4)){
            if(key==0){
                fprintf(fout, "_refln.F_meas_au\n" );
            }else if(key==1){
                value=get_float_after_id(str,item);
                fprintf(fout, "%8.2f ",value );
            }
        }else if(!strncmp_case(token[i], "SIGMA", 5)){
            if(key==0){
                fprintf(fout, "_refln.F_meas_sigma_au \n" );
            }else if(key==1){
                value=get_float_after_id(str,item);
                fprintf(fout, "%6.2f ",value );
            }
        }else if(!strncmp_case(token[i], "TEST", 4)){
            if(key==0){
                fprintf(fout, "_refln.status \n" );
            }else if(key==1){
                n=get_int_after_id(str,item);
                if(n==0){
                    fprintf(fout, " o ");
                }else if(n==1){
                    fprintf(fout, " f ");
                }else
                    fprintf(fout, " o ");
            }
            
       }else if(!strncmp_case(token[i], "IOBS", 4)){
            if(key==0){
                fprintf(fout, "_refln.intensity_meas\n" );
            }else if(key==1){
                value=get_float_after_id(str,item);
                fprintf(fout, "%10.2f ",value );
            }
        }else if(!strncmp_case(token[i], "SIGI", 4)){
            if(key==0){
                fprintf(fout, "_refln.intensity_sigma \n" );
            }else if(key==1){
                value=get_float_after_id(str,item);
                fprintf(fout, "%8.2f ",value );
            }

        }else if(!strncmp_case(token[i], "PHIB", 4)){
            if(key==0){
                fprintf(fout, "_refln.phase_meas \n" );
            }else if(key==1){
                value=get_float_after_id(str,item);
                fprintf(fout, "%8.2f ",value );
            }

        }else if(!strncmp_case(token[i], "FOM", 3)){
            if(key==0){
                fprintf(fout, "_refln.fom \n" );
            }else if(key==1){
                value=get_float_after_id(str,item);
                fprintf(fout, "%8.2f ",value );
            }


        }else if(!strncmp_case(token[i], "FC", 2)){
            if(key==0){
                fprintf(fout, "_refln.F_calc \n" );
            }else if(key==1){
                value=get_float_after_id(str,item);
                fprintf(fout, "%8.2f ",value );
            }

        }else if(!strncmp_case(token[i], "PHIC", 4)){
            if(key==0){
                fprintf(fout, "_refln.phase_calc \n" );
            }else if(key==1){
                value=get_float_after_id(str,item);
                fprintf(fout, "%8.2f ",value );
            }


        }else if(!strncmp_case(token[i], "HLA", 3)){
            if(key==0){
                fprintf(fout, "_refln.pdbx_HLA \n" );
            }else if(key==1){
                value=get_float_after_id(str,item);
                fprintf(fout, "%8.2f ",value );
            }

        }else if(!strncmp_case(token[i], "HLB", 3)){
            if(key==0){
                fprintf(fout, "_refln.pdbx_HLB \n" );
            }else if(key==1){
                value=get_float_after_id(str,item);
                fprintf(fout, "%8.2f ",value );
            }

        }else if(!strncmp_case(token[i], "HLC", 3)){
            if(key==0){
                fprintf(fout, "_refln.pdbx_HLC \n" );
            }else if(key==1){
                value=get_float_after_id(str,item);
                fprintf(fout, "%8.2f ",value );
            }

        }else if(!strncmp_case(token[i], "HLD", 3)){
            if(key==0){
                fprintf(fout, "_refln.pdbx_HLD \n" );
            }else if(key==1){
                value=get_float_after_id(str,item);
                fprintf(fout, "%8.2f ",value );
            }
            
        }
        
    }
}


void check_error_message(int argc, char **argv)
{
    int key=0, i, j, sf=0;
    const char *option[]=
        {"-i","-o", "-sf", "-out", "-pdb", "-man", "-flag", "-sf_type",
          "-pdb_id","-format", "-mtzdmp", "-cns_man", "END"
        };
    
    
    
    if(argc==1 || (argc==2 && !strcmp_case(argv[1], "-h"))) usage();
    
    if(argc<2){
        printf("\nInput error! There are not enough arguments.\n");
        usage();
    }
    
    for(i=1; i<argc; i++){
            /*
        if( argv[i-1][0]=='-' && argv[i][0]=='-'){
            printf("\nError! Both input arguments (%s ; %s) can not start with '-'!\n\n",
                   argv[i-1],argv[i]);
            printf("Please type 'sf_convert -h' for help!\n\n");
            exit(0);
        }
            */
        
        if(!strcmp_case(argv[i],"-sf")) {
            sf=1;
        }
        
        key=0;
        if( argv[i][0]=='-' ){
            for(j=0; j<100; j++){
                if(!strcmp(option[j], "END")) break;
                if(!strcmp_case(option[j], argv[i])) {
                    key=1;
                    break;
                }
            }
        }
        if(key==0 &&  argv[i][0]=='-' ){
            printf("\nThe input option (%s) is wrong!\n\n", argv[i]);
            printf("Please type 'sf_convert -h' for help!\n\n");
//            exit(0);
        }
    }
    if(sf==0){
        printf("\nError! you did not give the option '-sf'\n\n");
        usage();
    }

}

void reformat_mtz_output(char *inpfile)
{
    char  **line;
    FILE  *fout=NULL;
    int i,j,nline;

    
    line=get_lines_from_file(inpfile, &nline);

    for(i=0; i<nline; i++){
        rid_of_front_space(line[i]);
        if(line[i][0]=='#'){
           strcpy(line[i],"#\n");
 	}
   }
    
    fout = fopen(inpfile,"w");
    for(i=0; i<nline; i++){    
        if(!strncmp(line[i],"data_", 5)){
            fprintf(fout, "%s", line[i]);
            fprintf(fout,"#\n%s\n%s\n%s\n#\n",
                    "_audit.revision_id      1_0 ",
                    "_audit.creation_date  ?",
                    "_audit.update_record    'Initial release' "
                    );
	     continue;
        }else if(i>0 && line[i][0]=='#' &&  line[i-1][0]=='#' ){
	     continue;
        }else if(!strncmp(line[i],"_audit.", 7)){
	     for(j=i; j<i+20; j++){
		if(line[j][0]=='#' || !strncmp(line[j],"_cell.", 6)||
		!strncmp(line[j],"loop_", 5)){
                    i=j;
                    break;
                 }
		continue;
              }
        }
	fprintf(fout, "%s", line[i]);
    }
    fprintf(fout,"#END OF REFLECTIONS\n");
    fclose(fout);
   if(nline>0)free_memory_2d(line, nline); 
}

void write_loop(FILE *ftmp, FILE *fout)
{
    int n=0, k=0;
    char str[512], str1[512];
    
    
    while (fgets(str, sizeof str, ftmp) != NULL) {
        if(strchr(str,'#')){
            continue;
        }else {
            n=0;
            fprintf(fout, "%s", str);
            while (fgets(str1, sizeof str1, ftmp) != NULL) {
                n++;
                if(strchr(str1,'#')){
                    k=1;
                    if(n>0) fprintf(fout, "#\n");
                    break;
                }
                
                fprintf(fout, "%s", str1);
            }
            
            
        }
        if(n>0 && k==1) break;
    }
}


void delete_tab(char *line)
/* rid of \n */
{
    long len=0, i=0;
    
    len = strlen(line);
    for(i=0; i<len; i++)  
        if(line[i]=='\t')
            line[i]=' ';
    if(iscntrl(line[i-1]))line[i-1]='\0';
}

void cifparse(char *iFile, char *oFile, int width)
// read & write
{
  string diags;
  CifFile * fobjR = NULL;

  
  CifParser* cifParserR = NULL;
  fobjR = new CifFile();
  cifParserR = new CifParser(fobjR, fobjR->GetVerbose());
  cifParserR->Parse(iFile, diags);
  delete(cifParserR);

  if (!diags.empty()) printf(" Diagnostics [ %s] \n", diags.c_str() );
  
  fobjR->Write(oFile, true);
  fobjR->Close();
  if (fobjR) delete fobjR;
  
}

