/* convert a structure factor file from various format to mmCIF format
   not include mmcif --> mmcif!
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/types.h>
#include <unistd.h>
#include "rcsb_math.h"
#include "sf_convert.h"
#include "util.h"
 
extern int TEST;
extern char *CCP4, *MTZ2CIF, *PATHTMP;
int NFIRST=0;

void xgen2mmcif(FILE *fout,  char *inpfile, char *type);
void multan2mmcif(FILE *fout,  char *inpfile, char *type);

   
void various_2_cif(char *inpfile,char *outfile,char *format_inp,
                   char *pdb_id, char *type)
{
    int n=0;
    FILE *finp=NULL, *fout =NULL;
     
    
    if((finp=fopen(inpfile,"r"))==NULL){
        open_file_error(inpfile,"various_2_cif");
    }
    fclose(finp);
    
    
    if((fout=fopen(outfile,"w"))==NULL){
        printf("Error, no written permission in directory (%s)!\n", PATHTMP);
        exit(0);
    }
    if(strlen(pdb_id)<1) strcpy(pdb_id, "mtz2cif");
    
    fprintf(fout, "data_%s\n", pdb_id);
    
    
    if(!strcmp_case(format_inp, "CNS") || !strcmp_case(format_inp, "CNX")||
       !strcmp_case(format_inp, "XPLOR")){
        cns2mmcif(fout,  inpfile);
        
    }else if (!strcmp_case(format_inp, "SHELX") ){
        n = is_small_mol_cif(inpfile);
        if(n>0){ //small molecule
            printf("The input format is small molecule CIF.\n");
            cif2mmcif(fout,inpfile);
        }else
            shelx2mmcif(fout,inpfile,type );
        
    }else if (!strcmp_case(format_inp, "XPREP")||!strcmp_case(format_inp, "SAINT") ){
        saint2mmcif(fout,inpfile);
        
    }else if (!strcmp_case(format_inp, "fi") ){
        
        fi2mmcif(fout,inpfile, NULL);
//convert reflection file (3I4,2F8.2,2F8.2,I4 for SHELX ) to CIF
        
    }else if(!strncmp_case(format_inp, "HKL",3)|| !strcmp_case(format_inp,"scalepack") ||
            !strcmp_case(format_inp, "3DSCALE") ) {
        hkl2mmcif(fout,inpfile);
        
    }else if(!strcmp_case(format_inp, "EPMR") || !strcmp_case(format_inp, "OTHER")) {
        epmr2mmcif(fout,inpfile, type);
        
    }else if (!strcmp_case(format_inp, "TNT")){
        tnt2mmcif(fout,inpfile);
        
    }else if (!strcmp_case(format_inp, "XSCALE") ||!strcmp_case(format_inp, "XDS") ){
        xscale2mmcif(fout,inpfile);

    }else if (!strcmp_case(format_inp, "MTZ")){
//        mtz2mmcif(inpfile, outfile);
        mtz2mmcif_multi(inpfile, outfile);
        reformat_mtz_output(outfile);
        
    }else if(!strcmp_case(format_inp, "DTREK") ) {
        dtrek2mmcif(fout,inpfile);
        
    }else if(!strcmp_case(format_inp, "MAIN") ) {
        main2mmcif(fout,inpfile);
        
    }else if(!strcmp_case(format_inp, "XTALVIEW") ) {
        xtalview2mmcif(fout,inpfile);
        
    }else if(!strcmp_case(format_inp, "X-GEN") || !strcmp_case(format_inp, "XENGEN") ) {
        xgen2mmcif(fout,inpfile, type);
        
    }else if(!strcmp_case(format_inp, "MULTAN") ) {
        multan2mmcif(fout,inpfile, type);
        
    }else if(!strcmp_case(format_inp, "CIF") ) {
        cif2mmcif(fout,inpfile);
        
    }else {
        printf("Sorry, the input format (%s) is wrong!\n",format_inp );
        usage();
    }
    if(strcmp_case(format_inp, "MTZ")) fprintf(fout, "#END OF REFLECTIONS\n");
    fclose(fout);

    check_mmcif_status(outfile);
    
    
}

void check_mmcif_status(char *outfile)
// if status is wrong, reverse it.
{
    char **lines=NULL;
    int i, nline, n=0, j=0, m,k,len;
    FILE *fout=NULL;
    
    lines=get_lines_from_file(outfile, &nline);
    for(i=0; i<nline-5; i++){ 
        if(strstr(lines[i+0],"loop_") &&
           strstr(lines[i+1],"_refln.") &&
           strstr(lines[i+2],"_refln.") ){
            n=1;
            j=i;
            
            break;
        }
        if(i>1000) break;
    }
    
    if(n<=0 || nline<50){
        printf("Error! conversion to mmcif was not successful!\n");
        if(nline>0) free_memory_2d(lines, nline);
        return;
    }

    n=0;
    m=0;
    for(i=j; i<nline; i++){  //get cif items
        if(strstr(lines[i], " o"))
            n++;
        else if(strstr(lines[i], " f"))
            m++;
    }
    if(m>n+100) { // more flag than observed, rewrite
        fout=fopen(outfile, "w");
        
        for(i=0; i<nline; i++){
            if(strstr(lines[i+0],"loop_") &&
               strstr(lines[i+1],"_refln.") &&
               strstr(lines[i+2],"_refln.") ){
                j=i;
                break;
            }
            fprintf(fout,"%s",lines[i]);
        }

        for(i=j; i<nline; i++){
            len=strlen(lines[i]);
            
            for(k=1; k<len; k++){
                if(lines[i][k]=='o' && lines[i][k-1] ==' ')
                    lines[i][k]='f';
                else if(lines[i][k]=='f' && lines[i][k-1] ==' ')
                    lines[i][k]='o';
            }
            
            fprintf(fout,"%s",lines[i]);
        }
        fclose(fout);
    }
    
}


int is_small_mol_cif(char *inpfile)
{
    char **lines;
    int nline, i, n=0;


    lines=get_lines_from_file(inpfile, &nline);
    
    
    for(i=0; i<nline-5; i++){  //get cif items
        rid_of_front_end_space(lines[i+0]);
        rid_of_front_end_space(lines[i+1]);
        rid_of_front_end_space(lines[i+2]);
        rid_of_front_end_space(lines[i+3]);
        
        if(!strncmp(lines[i+0],"_refln_",7) &&
           !strncmp(lines[i+1],"_refln_",7) &&
           !strncmp(lines[i+2],"_refln_",7) &&
           !strncmp(lines[i+3],"_refln_",7)){
            n=1;
            break;
        }
        if(i>1000) break;
        
    }

    if(nline>0) free_memory_2d(lines, nline);
    return n;
}

void audit_sf_header(FILE *fout)
{
    fprintf(fout,"#\n%s\n%s\n%s\n#\n",
            "_audit.revision_id      1_0 ",
            "_audit.creation_date  ?",
            "_audit.update_record    'Initial release' "
            );
}

void cns2mmcif(FILE *fout,  char *inpfile)
/* Convert the CNS reflection data to mmCIF */
{
    char str[512],  tmp[512],  *flag_cif=NULL ;
    long  i, in=0,n=0, fn=0,  nref=0, H, K, L, nfg=0;
    float  F, sF, I, sI, pa,pb, pc, pd,ffg;
    FILE *finp, *ftmp;
    
    if((finp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "cns2mmcif");
    }
    
    ftmp = tmpfile();
    while (fgets(str, sizeof str, finp) != NULL) {
        sscanf(str,"%s",tmp);
        if(strstr_case(tmp, "DECLare")){
            if(pattern5(str,"DECLare", "NAME=IOBS", "DOM", "", "") ){
                in=1;
            }else if(pattern5(str,"DECLare", "NAME=FOBS", "DOM", "", "") ||
                     pattern5(str,"DECLare", "NAME=F_NAT", "DOM", "", "")){
                fn=1;
            }
        }else{
            if(strstr_case(str, " IOBS=") || strstr_case(str, " I_")){
                in=1;
            }
            if(strstr_case(str, " FOBS=")|| strstr_case(str, " F_") ){
                fn=1;
            }
            
            if(strstr_case(tmp, "INDE")){ //merge lines for the same INDEX
                fprintf(ftmp, "\n");
                nref++;
            }
            n=strlen(str);
            for(i=0; i<n; i++){
                if(isspace(str[i])) str[i]=' '; 
                fprintf(ftmp, "%c", str[i]);
            }
        }
    }
    
    if(nref < 50  || (in ==0 && fn ==0) ){
        printf("Error! Can not find IOBS or FOBS in your file (%s).\n", inpfile);
        return;
    }
    
    flag_cif = new char[nref+10];

    nfg=0;
    rewind(ftmp);
    while (fgets(str, sizeof str, ftmp) != NULL) {
        
        if(strstr_case(str,"INDE")){
            
            if(strstr_case(str, "TEST=") ){
                strcpy(tmp, strstr_case(str, "TEST=")+5);
                sscanf(tmp,"%f", &ffg);
                
                (ffg==1) ? flag_cif[nfg] = 'f' : flag_cif[nfg] = 'o';
                nfg++;
                
            }else if (strstr_case(str, "FREE=") ) {
                strcpy(tmp, strstr_case(str, "FREE=")+5);
                
                sscanf(tmp,"%f", &ffg);
                (ffg==1) ? flag_cif[nfg] = 'f' : flag_cif[nfg] = 'o';
                nfg++;
               
            }
        }
    }
    
    if(in>0 &&  fn>0 ){ //both exist   
        if(nfg>0)
            sf_data_header(fout, 0, 1);
        else
            sf_data_header(fout, 0, 0);
    }else if(in>0 &&  fn==0 ){ //only intensity exists        
        if(nfg>0)
            sf_data_header(fout, 2, 1);
        else
            sf_data_header(fout, 2, 0);
    }else if(fn>0 &&  in==0 ){ //only amplitude exists        
        if(nfg>0)
            sf_data_header(fout, 1, 1);
        else
            sf_data_header(fout, 1, 0);
    }        
    
    i=0;
    rewind(ftmp);
    while (fgets(str, sizeof str, ftmp) != NULL) { 
        if(strstr_case(str,"INDE")){
            F=0;
            sF=0;
            I=0;
            sI=0;
            sscanf(str,"%*s %ld%ld%ld",&H,&K,&L);

            
            if((strstr_case(str, "PA_MOD=") && strstr_case(str, "PB_MOD=") &&
                strstr_case(str, "PC_MOD=") && strstr_case(str, "PD_MOD="))||
               
               (strstr_case(str, "PA_") && strstr_case(str, "PB_") &&
                strstr_case(str, "PC_") && strstr_case(str, "PD_"))||
               
               (strstr_case(str, "PA=") && strstr_case(str, "PB=") &&
                strstr_case(str, "PC=") && strstr_case(str, "PD="))
               ){
                if(i==0){
                    fprintf(fout,"_refln.pdbx_HL_A_iso\n");
                    fprintf(fout,"_refln.pdbx_HL_B_iso\n");
                    fprintf(fout,"_refln.pdbx_HL_C_iso\n");
                    fprintf(fout,"_refln.pdbx_HL_D_iso\n");
                }
            }
            
            if(strstr_case(str, " FOBS=") && strstr_case(str, " IOBS=")){//both I&F
                
                strcpy(tmp, strstr_case(str, "FOBS=")+5);
                if(strlen(tmp)>0) sscanf(tmp,"%f", &F);

                if(!strstr(str, "SIGMA=")){
                    printf("Sorry! the Sigma for FOBS item is not SIGMA\n");
                    printf("Please change it to SIGMA and try again.\n");
                    return;
                }
                    
                strcpy(tmp, strstr_case(str, "SIGMA=")+6);
                if(strlen(tmp)>0) sscanf(tmp,"%f", &sF);
                
                strcpy(tmp, strstr_case(str, "IOBS=")+5);
                if(strlen(tmp)>0) sscanf(tmp,"%f", &I);
                
                if(strstr(str, "SIGI=")){
                    strcpy(tmp, strstr_case(str, "SIGI=")+5);
                    sscanf(tmp,"%f", &sI);
                }else if(strstr(str, "ISIGMA=")){
                    strcpy(tmp, strstr_case(str, "ISIGMA=")+7);
                    sscanf(tmp,"%f", &sI);
                    
                }else if(strstr(str, "SIGMAI=")){
                    strcpy(tmp, strstr_case(str, "SIGMAI=")+7);
                    sscanf(tmp,"%f", &sI);
                }else{
                    printf("Sorry! the Sigma data item is not SIGI or ISIGMA or SIGMAI.\n");
                    printf("Please change it to either of them, and try again.\n");
                    return;
                }
                
                if(nfg>0){
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %10.2f %10.2f %10.2f %c ", 
                            H,K,L, F, sF, I, sI, flag_cif[i]);
                }else
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %10.2f %10.2f %10.2f ", 
                            H,K,L, F, sF, I, sI);
                i++;
                
            }else if(!strstr_case(str, " FOBS=") && strstr_case(str, " IOBS=")){ //only I
                
                strcpy(tmp, strstr_case(str, "IOBS=")+5);
                sscanf(tmp,"%f", &I);
                
                if(strstr_case(str, "SIGI=")){
                    strcpy(tmp, strstr_case(str, "SIGI=")+5);
                    sscanf(tmp,"%f", &sI);
                    
                }else if(strstr_case(str, "ISIGMA=")){
                    strcpy(tmp, strstr_case(str, "ISIGMA=")+7);
                    sscanf(tmp,"%f", &sI);
                    
                }else if(strstr_case(str, "SIGMAI=")){
                    strcpy(tmp, strstr_case(str, "SIGMAI=")+7);
                    sscanf(tmp,"%f", &sI);
                    
                }else 
                    sI=0;
                
                 if(nfg>0){
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %10.2f %c ", 
                            H,K,L, I, sI, flag_cif[i]);
                }else
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %10.2f ", 
                            H,K,L, I, sI);
               
                i++;
                
            }else if(strstr_case(str, " FOBS=") && !strstr_case(str, " IOBS=")){//only F
                
                strcpy(tmp, strstr_case(str, "FOBS=")+5);
                sscanf(tmp,"%f", &F);
                
                if(strstr_case(str, "SIGMA=")){
                    strcpy(tmp, strstr_case(str, "SIGMA=")+6);
                    sscanf(tmp,"%f", &sF);
                }else 
                    sF=0;

                 if(nfg>0){
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %10.2f  %c ", 
                            H,K,L, F, sF, flag_cif[i]);
                }else
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %10.2f ", 
                            H,K,L, F, sF);
                i++;
            }else if(strstr_case(str, " F_NAT") && !strstr_case(str, " IOBS=")){//only F
                
                strcpy(tmp, strstr_case(str, "F_NAT")+6);
                sscanf(tmp,"%f", &F);
                
                if(strstr_case(str, "S_NAT=")){
                    strcpy(tmp, strstr_case(str, "S_NAT=")+6);
                    sscanf(tmp,"%f", &sF);
                }else 
                    sF=0;

                 if(nfg>0){
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %10.2f  %c ", 
                            H,K,L, F, sF, flag_cif[i]);
                }else
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %10.2f ", 
                            H,K,L, F, sF);
                i++;
                
            }else if(strstr_case(str, " F_1") && !strstr_case(str, " IOBS=")){//only F
                
                strcpy(tmp, strstr_case(str, "F_1")+6);
                sscanf(tmp,"%f", &F);
                
                if(strstr_case(str, "S_1=")){
                    strcpy(tmp, strstr_case(str, "S_1=")+4);
                    sscanf(tmp,"%f", &sF);
                }else 
                    sF=0;

                 if(nfg>0){
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %10.2f  %c ", 
                            H,K,L, F, sF, flag_cif[i]);
                }else
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %10.2f ", 
                            H,K,L, F, sF);
                i++;
            }
            
            if(strstr_case(str, "PA_MOD=") && strstr_case(str, "PB_MOD=") &&
               strstr_case(str, "PC_MOD=") && strstr_case(str, "PD_MOD=")){
            
                strcpy(tmp, strstr_case(str, "PA_MOD=")+7);
                sscanf(tmp,"%f", &pa);
                strcpy(tmp, strstr_case(str, "PB_MOD=")+7);
                sscanf(tmp,"%f", &pb);
                strcpy(tmp, strstr_case(str, "PC_MOD=")+7);
                sscanf(tmp,"%f", &pc);
                strcpy(tmp, strstr_case(str, "PD_MOD=")+7);
                sscanf(tmp,"%f", &pd);
                fprintf(fout," %8.3f  %8.3f  %8.3f  %8.3f \n", pa, pb, pc, pd);
                
            }else if(strstr_case(str, "PA=") && strstr_case(str, "PB=") &&
               strstr_case(str, "PC=") && strstr_case(str, "PD=")){
            
                strcpy(tmp, strstr_case(str, "PA=")+3);
                sscanf(tmp,"%f", &pa);
                strcpy(tmp, strstr_case(str, "PB=")+3);
                sscanf(tmp,"%f", &pb);
                strcpy(tmp, strstr_case(str, "PC=")+3);
                sscanf(tmp,"%f", &pc);
                strcpy(tmp, strstr_case(str, "PD=")+3);
                sscanf(tmp,"%f", &pd);
                fprintf(fout," %8.3f  %8.3f  %8.3f  %8.3f \n", pa, pb, pc, pd);
                
            }else if(strstr_case(str, "PA_1=") && strstr_case(str, "PB_1=") &&
               strstr_case(str, "PC_1=") && strstr_case(str, "PD_1=")){
            
                strcpy(tmp, strstr_case(str, "PA_1=")+5);
                sscanf(tmp,"%f", &pa);
                strcpy(tmp, strstr_case(str, "PB_1=")+5);
                sscanf(tmp,"%f", &pb);
                strcpy(tmp, strstr_case(str, "PC_1=")+5);
                sscanf(tmp,"%f", &pc);
                strcpy(tmp, strstr_case(str, "PD_1=")+5);
                sscanf(tmp,"%f", &pd);
                fprintf(fout," %8.3f  %8.3f  %8.3f  %8.3f \n", pa, pb, pc, pd);
                
            }else if(strstr_case(str, "PA_CONV=") && strstr_case(str, "PB_CONV=") &&
               strstr_case(str, "PC_CONV=") && strstr_case(str, "PD_CONV=")){
            
                strcpy(tmp, strstr_case(str, "PA_CONV=")+8);
                sscanf(tmp,"%f", &pa);
                strcpy(tmp, strstr_case(str, "PB_CONV=")+8);
                sscanf(tmp,"%f", &pb);
                strcpy(tmp, strstr_case(str, "PC_CONV=")+8);
                sscanf(tmp,"%f", &pc);
                strcpy(tmp, strstr_case(str, "PD_CONV=")+8);
                sscanf(tmp,"%f", &pd);
                fprintf(fout," %8.3f  %8.3f  %8.3f  %8.3f \n", pa, pb, pc, pd);
                
                
            }else{
                fprintf(fout,"\n");
            }
                
        }
        
    }
    if(nref>0) delete [] flag_cif;
    if(finp) fclose(finp);
    if(ftmp) fclose(ftmp);
    
}

void check_flag_num(float *flag, long nref, const char *id, char *flag_cif)
{
    long i, nf=0, no=0, yes=0;

    for(i=0; i<nref; i++){
        if(!strcmp_case(id,"CNS")){
            if(flag[i] > 0.5 )
                nf++;
            else 
                no++;
        }else if(!strcmp_case(id,"SHELX")){
            if(flag[i] < -0.5 )
                nf++;
            else 
                no++;
        }
    }

    if(nf>10 && no >50){
//            printf("%7ld, %7ld, flag=%.3lf\n",nf, no, float(nf)/float(no));
        
        if(float(nf)/float(no) > 0.03){
            yes = 1;
        }else{
            yes = 0;
        }
    }else{
        yes = 0;
    }

    if(yes==1){ // use flag in the file
        for(i=0; i<nref; i++){
            flag_cif[i]='o';
            if(!strcmp_case(id,"CNS")){
                if(flag[i] > 0.5 )
                    flag_cif[i] = 'f';
                else 
                    flag_cif[i] = 'o';
            }else if(!strcmp_case(id,"SHELX")){
                
                if(flag[i] < -0.5 )
                    flag_cif[i] = 'f';
                else
                    flag_cif[i] = 'o';
            }
        }
        
    }else{ // generate 10% 
        

        for(i=0; i<nref; i++){
            flag_cif[i]='o';
        }

    }
}

void formated_4_8(char *str, int *h, int *k, int *l, float *a,float *sa)
{
    char tmp[10];
    int  H, K, L;
    float sI=0 , I;
    
    strncpy(tmp, str+0, 4);
    tmp[4] = '\0';
    sscanf(tmp, "%d", &H);
        
    strncpy(tmp, str+4, 4);
    tmp[4] = '\0';
    sscanf(tmp, "%d", &K);

    strncpy(tmp, str+8, 4);
    tmp[4] = '\0';
    sscanf(tmp, "%d", &L);

    strncpy(tmp, str+12, 8);
    tmp[8] = '\0';
    sscanf(tmp, "%f", &I);

    if(strlen(str)>20){  
        strncpy(tmp, str+20, 8);
        tmp[8] = '\0';
        sscanf(tmp, "%f", &sI);
    }
        
    *h=H; *k=K; *l=L; *a=I; *sa=sI;   
}

void shelx2mmcif(FILE *fout,  char *inpfile, char *type)
/*convert SHELX reflection file (3I4,2F8.2,I4 for SHELX ) to CIF format
*/
{
    char  str[512],  tmp[10];
    int  H, K, L, n=0, i, flag=0;
    float sI , I;
    FILE *fp;

    if((fp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "shelx2mmcif");
    }
    
    i=0;
    flag=0;
    while (fgets(str, sizeof str, fp) != NULL) { // find Rfree flag
        n=0;
        sscanf(str + 28, "%d", &n);
        if(n==1 || n== -1) flag=1;
        if(i++>500) break;
    }

    if(!strcmp(type,"F")){
        (flag==1)? sf_data_header(fout, 1, 1) : sf_data_header(fout, 1, 0);
    }else if(!strcmp(type,"I")){
        (flag==1)? sf_data_header(fout, 2, 1) : sf_data_header(fout, 2, 0);
    }else{
        printf("You SF type is assumed to I. If not, please give -sf_type ? \n");
        (flag==1)? sf_data_header(fout, 2, 1) : sf_data_header(fout, 2, 0);
    }
    
    
    n=0;
    rewind(fp);
    while (fgets(str, sizeof str, fp) != NULL) {
        sI = 0;
        if(strlen(str)<20)continue;

        formated_4_8(str, &H, &K, &L, &I, &sI);
        
        strncpy(tmp, str+28, 4);
        tmp[4] = '\0';
        sscanf(tmp, "%d", &flag);
        if(H==0 && K==0 && L==0) continue;
        
        if(flag==1)
            fprintf(fout,"1 1 1 %4d %4d %4d %10.2f %10.2f o\n",  H, K, L, I, sI);
        else if(flag==-1){
            fprintf(fout,"1 1 1 %4d %4d %4d %10.2f %10.2f f\n",  H, K, L, I, sI);
        }else if(flag==0){
            fprintf(fout,"1 1 1 %4d %4d %4d %10.2f %10.2f\n",  H, K, L, I, sI);
        }      
    }
    fclose(fp);    
}

void saint2mmcif(FILE *fout,  char *inpfile)
/*convert SAINT/XPREP reflection file  to CIF format
*/
{
    char  str[512];
    int  H, K, L, n=0;
    float sI=0 , I;
    FILE *fp;

    if((fp = fopen(inpfile, "r"))==NULL) open_file_error(inpfile, "saint2mmcif");
    
    n=0;
    sf_data_header(fout, 2, 0);
    while (fgets(str, sizeof str, fp) != NULL) {
        formated_4_8(str, &H, &K, &L, &I, &sI);
        fprintf(fout,"1 1 1 %4d %4d %4d %10.2f %10.2f\n",  H, K, L, I, sI);
        n++;
    }
    fclose(fp);
    if(n<10) {
        printf("Error! conversion to mmcif failed\n");
        return;
    }
}

void fi2mmcif(FILE *fout,  char *inpfile, char *type)
/*convert reflection file (3I4,2F8.2,2F8.2,I4 for SHELX ) to CIF format
here F and I exist also with status
*/
{
    char  str[512],  tmp[10];
    int  H, K, L,  n=0, flag=0;
    float sI , I, sF , F;
    
    FILE *fp;

    if((fp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "fi2mmcif");
    }
    
    sf_data_header(fout, 0, 1);
    
    n=0;
    rewind(fp);
    while (fgets(str, sizeof str, fp) != NULL) {
        n++;
        
        sI = 0;
        if(strlen(str)<20)continue;
        
        formated_4_8(str, &H, &K, &L, &F, &sF);
        
        strncpy(tmp, str+28, 8);
        tmp[8] = '\0';
        sscanf(tmp, "%f", &I);
        
        strncpy(tmp, str+36, 8);
        tmp[8] = '\0';
        sscanf(tmp, "%f", &sI);
        
        strncpy(tmp, str+44, 4);
        tmp[4] = '\0';
        sscanf(tmp, "%d", &flag);

        if(flag==1){
            fprintf(fout,"1 1 1 %4d %4d %4d %8.2f %6.2f %8.2f %7.2f f\n",  H, K, L, F, sF, I, sI);
        }else {
            fprintf(fout,"1 1 1 %4d %4d %4d %8.2f %6.2f %8.2f %7.2f o\n",  H, K, L, F, sF, I, sI);
        }      
    }
    fprintf(fout,"#END OF REFLECTIONS\n");
    

    fclose(fp);    
}

void tnt2mmcif(FILE *fout,  char *inpfile)
/*convert TNT reflection file (A4, 4I4,2F8.2, F8.2,F8.4 ) to CIF format
  HKL, H, K, L,  F, sigmaF, phase, FOM in each row
*/
{

    char str[256], tmp[256];
    int H, K, L;
    float  F, sF;
    FILE *fp;

    if((fp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "tnt2mmcif");
    }
    
    printf("\nThe reflection data type is assumed to be amplitude (F)!\n");
    sf_data_header(fout, 1, 0);

    while (fgets(str, sizeof str, fp) != NULL){
        sF = 0;
        upperstr(str);
        if(strncmp(str, "HKL ",4))continue;
        if(strlen(str)<23)continue;
        strcpy(tmp, str+4);
        
        formated_4_8(tmp, &H, &K, &L, &F, &sF);
        fprintf(fout,"1 1 1 %4d %4d %4d %9.1f %7.1f\n",H, K, L, F, sF);
    }

    fclose(fp);
    
}

void xtalview2mmcif(FILE *fout,  char *inpfile)
/* this is a .fin format only !!!
File contains: h k l f1 sigma(f1) f2 sigma(f2). The f1 and f2 can be
Bijvoet pairs, fp and fph, or any other pairable data. If one of the
two observations is missing, then f should equal 0.0
and sigma(f) should equal 9999.0    
*/
{
    float  F, sF, Fp, sFp;
    char str[256], data_type[10];
    int H, K, L, m=0;
    
    FILE *fp;

    if((fp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "xtalview2mmcif");
    }

    
    strcpy(data_type, "F");
    printf("\nThe input reflection data type is assumed to be amplitude (F)!\n");
    
    sf_data_header(fout, 4, 0);
    
    while (fgets(str, sizeof str, fp) != NULL){
        m=sscanf(str, "%d %d %d  %f %f %f %f \n", &H,&K, &L, &F,&sF, &Fp, &sFp);
        if(m!=7) continue;

        if(F<0.01 && sF>9998){
            fprintf(fout,"1 1 1 %4d %4d %4d  ?  ? %8.2f %6.2f \n",
                    H, K, L,  Fp, sFp);
        }else if(Fp<0.01 && sFp>9998){
            fprintf(fout,"1 1 1 %4d %4d %4d  %8.2f %6.2f ?  ? \n",
                    H, K, L, F, sF);
        }else{
            fprintf(fout,"1 1 1 %4d %4d %4d  %8.2f %6.2f %8.2f %6.2f \n",
                    H, K, L, F, sF,  Fp, sFp);
        }
        
    }
    fclose(fp);
}


void xgen2mmcif(FILE *fout,  char *inpfile, char *type)
/*convert X-gen reflection file  to CIF format
h, k, l, res, F, sig(F), F+, sig(F+), F-, sig(F-), iflag
( 3I4, 1X, F6.4, 6(1X, F8.2) 1X, I2 )   separated by a space
X-GEN and  XENGEN are the same.
*/
{
    float F=0, sF=0,  Fp=0, sFp=0,  Fm=0, sFm=0;
    char str[512];
    int H, K, L, n, m=0;
    FILE *fp;

    if((fp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "xgen2mmcif");
    }
    
    if(!strcmp(type,"I")){
        sf_data_header(fout, 3, 0);
        fprintf(fout, "_refln.intensity_meas\n" );
        fprintf(fout, "_refln.intensity_sigma\n" );
        printf("\nThe reflection data type is assumed to be amplitude (I)!\n");
    }else{
        sf_data_header(fout, 4, 0);
        fprintf(fout, "_refln.F_meas_au\n" );
        fprintf(fout, "_refln.F_meas_sigma_au \n" );
        printf("\nThe reflection data type is assumed to be amplitude (F)!\n");
    }
    n=0;
    while (fgets(str, sizeof str, fp) != NULL){
        if(strlen(str)<61)continue;
        m=sscanf(str, "%d %d %d %*s %f %f %f %f %f %f\n",
               &H,&K, &L,&F,&sF, &Fp, &sFp, &Fm, &sFm);
        if (m<9) continue;
        
        fprintf(fout,"1 1 1 %3d %3d %3d  %8.2f %6.2f %8.2f %6.2f  %8.2f %6.2f \n",
                H, K, L, Fp, sFp,  Fm, sFm, F, sF);
        n++;
        
    }
    fclose(fp);
    
}
void multan2mmcif(FILE *fout,  char *inpfile, char *type)
/*
The file has h, k, l, f, imt in FORMAT(3I4,7X,F7.0,I6),
where imt=0 for a good reflection.  
*/
{
    char str[256];
    int H, K, L, m=0;
    float F=0;
    FILE *fp;

    if((fp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "multan2mmcif");
    }
    if(!strcmp(type,"I")){
        sf_data_header(fout, 2, 0);
        printf("Input data type is intensity (I)!\n");
    }else{
        sf_data_header(fout, 1, 0);
        printf("Input data type is amplitude (F)!\n");
    }
    
     
    while (fgets(str, sizeof str, fp) != NULL){
        if(strlen(str)<30)continue;
        m=sscanf(str, "%d %d %d  %f \n", &H,&K, &L,&F);
        if (m<4) continue;
        fprintf(fout,"1 1 1 %3d %3d %3d  %8.2f ? \n", H, K, L, F);
        
    }
    fclose(fp);

}


void main2mmcif(FILE *fout,  char *inpfile)
/*convert main reflection file (INDEX,FOBS,SIGMA,TEST ) to CIF format
The default format "main", which assumes keyword based records
based on old X-plor version format.
INDEX    8     1   -20  FOBS  1262.150  SIGMA 344.3 PHA 132.23 WE 0.345 TEST 0
http://www-bmb.ijs.si/
*/
{
    char status='o', str[256], **line;
    float  F=0, sF=0, pha=0, we=0;
    int i, H=0, K=0, L=0, n=0,n1=0,n2=0, phase=0, test=0, nstr=0;
    FILE *fp;


    if((fp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "main2mmcif");
    }

    n=0;
    while (fgets(str, sizeof str, fp) != NULL){
        if(pattern5(str, "IND", " FOB", " SIG",".", " TEST")){
            test=1;
        }else if(pattern5(str,"IND", " FOB", " SIG"," PHA", " WE")){
            phase=1;
        }
        if(pattern5(str,"IND", " FOB", " SIG", ".", "") && n++>40) break;
    }
    
    (test)?sf_data_header(fout, 1, 1):sf_data_header(fout, 1, 0);
    if(phase){
        fprintf(fout, "_refln.phase_meas\n");        
        fprintf(fout, "_refln.fom\n");              
    }
            
    if (n<10){
        printf("Error! Too few reflection data.\n");
        fclose(fp);   
        return;
    }
    
    rewind(fp);
    while (fgets(str, sizeof str, fp) != NULL){
        sF = 0;
        nstr=0;
        upperstr(str);
        line = string_token(str, " " , &nstr);
        for(i=0; i<nstr; i++){
            H=0; K=0; L=0;
            if(strstr(line[i], "IND")){
                H=atoi(line[i+1]);
                K=atoi(line[i+2]);
                L=atoi(line[i+3]);
                i=i+2;
                n1=1;
            }else if(strstr(line[i], "FOB")){
                F=atof(line[i+1]);
                n2=1;
            }else if(strstr(line[i], "SIG")){
                sF=atof(line[i+1]);
            }else if(strstr(line[i], "PHA")){
                pha=atof(line[i+1]);
            }else if(strstr(line[i], "WE")){
                we=atof(line[i+1]);
            }else if(strstr(line[i], "TEST")){
                (atoi(line[i+1])>0)? status='f' : status='o';
            }
        }
        if(!(n1 && n2) || (H==0 && K==0 && L==0) ) continue;
        if(test ==1){
            fprintf(fout,"1 1 1 %4d %4d %4d %8.2f %7.2f %c", H, K, L,  F, sF, status);
        }else{
            fprintf(fout,"1 1 1 %4d %4d %4d %8.2f %7.2f ", H, K, L,  F, sF);
        }
        if(phase)fprintf(fout," %6.1f %6.3f", pha, we);
        fprintf(fout,"\n");
        if(nstr) free_memory_2d(line, nstr);
        
    }
    fclose(fp);   
}

void epmr2mmcif(FILE *fout,  char *inpfile,char *type)
/*convert EPMR reflection file (free format ) to CIF format
   H, K, L,  Fo,  in each row
*/
{

    float  F, sF=0, status;
    char  str[512], ch;
    int H, K, L, n, nref=0;
    FILE *fp;


    if((fp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "epmr2mmcif");
    }

                   
    while (fgets(str, sizeof str, fp) != NULL){
        
        if(strlen(str)<20)continue;
        n=sscanf(str,"%d%d%d%f%f%f", &H, &K, &L, &F, &sF ,&status);
        
        if(n==4 || n==5 || n==6 )
            nref++;
        
        if(nref==1 && (n==4 || n==5)){
            if(!strcmp(type, "F")){
                
                printf("\nThe reflection data type is  amplitude (F)!\n");
                sf_data_header(fout, 1, 0);
            }else if(!strcmp(type, "I")){
                
                printf("\nThe reflection data type is intensity (I)!\n");
                sf_data_header(fout, 2, 0);
            }else
                sf_data_header(fout, 1, 0);
        }else if(nref==1 &&  n==6){
            if(!strcmp(type, "F")){
                printf("\nThe reflection data type is amplitude (F)!\n");
                sf_data_header(fout, 1, 1);
            }else if(!strcmp(type, "I")){
                printf("\nThe reflection data type is intensity (I)!\n");
                
                sf_data_header(fout, 2, 1);
            }else
                sf_data_header(fout, 1, 1);
        }
        
        
        if(n==4){
            fprintf(fout,"1 1 1 %4d %4d %4d %10.2f 0.0 \n", H, K, L, F );
        }else if (n==5){
            fprintf(fout,"1 1 1 %4d %4d %4d %10.2f %8.2f\n", H, K, L, F, sF );
        }else if (n==6){
            if(status<1)
                ch='f';
            else
                ch='o';
                
            fprintf(fout,"1 1 1 %4d %4d %4d %10.2f %8.2f %c\n", H, K, L, F, sF, ch );
        }else
            continue;
    }

    fclose(fp);
}

void cryst_infor(FILE *fout, float *a, char *sym,int num)
{
    char symmetry[20];
    
    fprintf(fout, "#\n");
    
    if(num>0)fprintf(fout, "_symmetry.Int_Tables_number  %d\n", num);
    if(strlen(sym)>1){        
        space_group_convertor(sym, symmetry); 
        fprintf(fout, "_symmetry.space_group_name_H-M  '%s'\n", symmetry);
    }
    if(a[0]>0.5 && a[2]>0.5){
        fprintf(fout, "#\n");
        fprintf(fout, "_cell.length_a  %.3f\n", a[0]);
        fprintf(fout, "_cell.length_b  %.3f\n", a[1]);
        fprintf(fout, "_cell.length_c  %.3f\n", a[2]);
        fprintf(fout, "_cell.angle_alpha  %.3f\n", a[3]);
        fprintf(fout, "_cell.angle_beta  %.3f\n", a[4]);
        fprintf(fout, "_cell.angle_gamma  %.3f\n", a[5]);
    }
    
}


void hkl2mmcif(FILE *fout, char *filename)
/* get the right format for HKL2000/scalepack
    
1, NO ANOMALOUS flag ----- h, k, l, I, sigma(I)  (3I4, 2F8.1) length=28
2, WITH ANOMALOUS flag --- h, k, l, I+, sigma(I+), I-, sigma(I-) ;
   (3I4, 4F8.1) length=44
3, NO MERGE  --- h, k, l, asymmetric unit #, I, sigma(I) ; (4I4,2F8.1)  
   ANOMALOUS flag has no effect.  length=32
4, NO MERGE original index --- original hkl, unique hkl, batch (film)
   number, a flag (0 = centric, 1 = I+, 2 = I-), another flag (0 =
   hkl reflecting above the spindle, 1 = hkl reflecting below the spindle), 
   the asymmetric unit of the reflection, I (scaled, Lorentz and Polarization
   corrected), and the sigma of I. The format is (6i4, i6, 2i2, i3, 2f8.1).
   (RAW data!!)  length=53

*/
    
{
    char str[512],symmetry_in[20],symmetry[20];
    int H, K, L, len=0, n;
    int n1=0,n2=0,n3=0,n4=0,nref=0;
    float a[10], ii=0, sii=0, I, sI ;
    FILE  *finp;
    
    if((finp = fopen(filename, "r"))==NULL) open_file_error(filename, "hkl2mmcif");

    nref=0;
    while (fgets(str, sizeof str, finp) != NULL) {
        rid_of_end_space(str);
        len=strlen(str);
        if(len>60 && pattern5(str,".",".", ".",".","")){
            n=sscanf(str,"%f%f%f%f%f%f",
                     &a[0],&a[1],&a[2],&a[3],&a[4],&a[5]);
        
            if(n==6 && a[0]>1.0 && a[2]>1.0){
                get_string(str, 60, 70, symmetry_in);
                space_group_convertor(symmetry_in,symmetry);
                cryst_infor(fout, a,symmetry,-1);
            }
        }else if(len==28 || len==29){ //I
            n1++;
        }else if(len==44 || len==45){ //I+, I-
            n2++;
        }else if(len==35 || len==36){
            n3++;
        }else if(len==53 || len==54){
            n4++;
        }
        if(nref++>3000)break;
    }
    
    printf("The input reflection data type is assumed to be intensity (I)!\n");
    (n2>10)?sf_data_header(fout,3, 0) : sf_data_header(fout,2, 0);
    
    nref=0;
    rewind(finp);
    while (fgets(str, sizeof str, finp) != NULL) {
        len=strlen(str);
        if(!(len==28 || len==29 || len==44 || len==45 ||
             len==35 || len==36 || len==53 || len==54))continue;
        
        nref++;
        
        if(n2>10){ //I+
            formated_4_8(str, &H, &K, &L, &I, &sI);
            if(len==28 || len==29)
                fprintf(fout,"1 1 1 %4d %4d %4d %9.1f %7.1f        0.0    -1.0\n",
                        H, K, L, I, sI);
            else{
                ii=get_float_by_length(str, 28, 35);
                sii=get_float_by_length(str, 36, 43);
                fprintf(fout,"1 1 1 %4d %4d %4d %9.1f %7.1f  %9.1f %7.1f\n",
                        H, K, L, I, sI, ii, sii);
            }
            
        }else if(n3>10){//NO MERGE
            H=get_int_by_length(str, 0, 3);
            K=get_int_by_length(str, 4, 7);
            L=get_int_by_length(str, 8, 11);
            
            I=get_float_by_length(str, 19, 26);
            sI=get_float_by_length(str, 27, 34);
            
            fprintf(fout,"1 1 1 %4d %4d %4d  %9.1f %7.1f\n", H, K, L,  I, sI);
            
        }else if(n4>10){//NO MERGE orin
            if(len<52) continue;
            
            H=get_int_by_length(str, 0, 3);
            K=get_int_by_length(str, 4, 7);
            L=get_int_by_length(str, 8, 11);
            
            I=get_float_by_length(str, 37, 44);
            sI=get_float_by_length(str, 45, 52);

            fprintf(fout,"1 1 1 %4d %4d %4d  %9.1f %7.1f\n", H, K, L,  I, sI);
            
        }else {
            formated_4_8(str, &H, &K, &L, &I, &sI);
            fprintf(fout,"1 1 1 %4d %4d %4d  %9.1f %7.1f\n", H, K, L,  I, sI);
        }
        
    }
    fclose(finp);
    
    if(!(n1<10 || n2<10  || n3<10  || n4<10 || nref<10) ){
        printf("Error! The HKL2000/SCALEPACK format is wrong!\n");
    }
}


void dtrek2mmcif(FILE *fout, char *filename)
/* write reflections from d*trek */
{
    char str[512];
    int h=0, k=0, l=0, m, n1=0, sg_num=0, nref=0;
    float i, sigmai, plus_i, plus_s, minus_i, minus_s, a[10];
    FILE *finp;

    if((finp = fopen(filename, "r"))==NULL) {
        open_file_error(filename, "dtrek2mmcif");
    }

    nref=0;
    while (fgets(str, sizeof str, finp) != NULL) {
        if(nref++>500) break;
        if(pattern5(str,"CRYSTAL_SPACEGROUP=", ";", "", "","")){
            sg_num=get_int(str, "=", ";");
        }else if(pattern5(str,"CRYSTAL_UNIT_CELL=", ";", "", "","")){
            sscanf(get_substr(str, "=", ";"), "%f %f %f %f %f %f",
                   &a[0],&a[1],&a[2],&a[3],&a[4],&a[5]);
        }        
        m=sscanf(str, " %d%d%d %f%f %f%f %f%f",
                 &h, &k, &l, &i, &sigmai, &plus_i, &plus_s, &minus_i, &minus_s);
        if(m==9)n1++;
            
    }
    if(a[0]>0 && a[1]>0 && a[5]>0) cryst_infor(fout, a,(char *)"?",sg_num);
        
    printf("The reflection data type is assumed to be intensity (I)!\n");
    if(n1>50){
        sf_data_header(fout,3, 0);
        fprintf(fout, "_refln.intensity_meas\n" );
        fprintf(fout, "_refln.intensity_sigma\n" );
    }else
        sf_data_header(fout,2, 0);

    rewind(finp);
    while (fgets(str, sizeof str, finp) != NULL) {
        m=sscanf(str, " %d%d%d %f%f %f%f %f%f",
                 &h, &k, &l, &i, &sigmai, &plus_i, &plus_s, &minus_i, &minus_s);
        
        if(m==5 && sigmai >=0){
            fprintf(fout,"1 1 1 %4d %4d %4d %9.2f %7.2f\n", h, k, l, i, sigmai);
        }else if(m==9){
            fprintf(fout,"1 1 1 %4d %4d %4d %9.2f %7.2f %9.2f %7.2f %9.2f %7.2f\n",
                    h, k, l, plus_i, plus_s, minus_i, minus_s, i, sigmai);
        }
    }
    
    fclose(finp);
    
}

void xscale2mmcif(FILE *fout, char *filename)
// XSCALE || XDS
{
    char **line;
    int i,h,k,l, nline,start=0, npair=0, sg_num=0;
    float a1,a2,a3,a4, b[6];
    
    line=get_lines_from_file(filename, &nline);
    
    for(i=0; i<nline-10; i++){
        if(pattern5(line[i],"!SPACE_GROUP_NUMBER=", "", "", "","")){
            sg_num=get_int_after_id(line[i], "=");
        }else if(pattern5(line[i],"!UNIT_CELL_CONSTANTS=", ".", ".", ".","")){
            sscanf(strstr(line[i], "=")+1, "%f %f %f %f %f %f",
                   &b[0],&b[1],&b[2],&b[3],&b[4],&b[5]);
        }
        
        if(strstr(line[i],"!END_OF_HEADER")) {
            if(b[0]>0 && b[1]>0 && b[5]>0) 
                cryst_infor(fout, b,(char *)"?",sg_num);
            
            start=i+1;
            break;
        }
        
        if(strstr_case(line[i], "!ITEM_H=1") &&
           strstr_case(line[i+1], "!ITEM_K=2") &&
           strstr_case(line[i+2], "!ITEM_L=3") &&
           strstr_case(line[i+3], "!ITEM_IOBS=4") &&
           strstr_case(line[i+4], "!ITEM_SIGMA(IOBS)=5") &&
           strstr_case(line[i+5], "!ITEM_IOBS'=6") &&
           strstr_case(line[i+6], "!ITEM_SIGMA(IOBS')=7"))
           npair=2;
    }

    if(npair==2){
        sf_data_header(fout, 3, 0);
        for(i=start; i<nline; i++){
            sscanf(line[i], "%d%d%d%f%f%f%f\n", &h,&k,&l, &a1, &a2, &a3, &a4);
            fprintf(fout,"1 1 1 %3d %3d %3d %9.2f %7.2f %9.2f %7.2f\n", h,k,l, a1, a2,a3,a4);
        }
    }else{
        sf_data_header(fout, 2, 0);
        for(i=start; i<nline; i++){
            if(npair==0){
                sscanf(line[i], "%d%d%d%f%f\n", &h,&k,&l, &a1, &a2);
                fprintf(fout,"1 1 1 %3d %3d %3d %9.2f %7.2f \n", h,k,l, a1, a2);
            }
        }
    }
     
    if(line && nline) free_memory_2d(line, nline);
}


void  cif2mmcif(FILE *fout, char *inpfile) 
// convert CIF (shelx) to mmCIF reflection items.
{
    char str[512], tmp[512], **line=NULL, **item=NULL, **lines=NULL ;
    int  index[80], i, j,n, nt, nitem_tot=0;
    int *line_index, nstr=0, nitem=0, nline=0,nline_tmp=0, start_hkl=0;
    float cell[6];
    FILE   *finp;

    
    if((finp=fopen(inpfile, "r")) == NULL){
        open_file_error(inpfile,"cif2mmcif");
    }

    for(i=0; i<80; i++){
        index[i]=-10;
    }
    
   
    nline_tmp=0;
    while(fgets(str, sizeof str, finp) != NULL){ //get # of lines 
        nline_tmp++;
    }

    lines = new char *[nline_tmp];
    line_index = new int[nline_tmp];
   
    rewind(finp);
    nline=0;
    nt=0;
    while(fgets(str, sizeof str, finp) != NULL){
        n=sscanf(str, "%s", tmp );
        if(n<=0) continue;
        
        
        if(!strcmp_case(tmp,"_cell_length_a")){
            cell[0] =get_float_after_id(str, "length_a");
        }else if(!strcmp_case(tmp,"_cell_length_b")){
            cell[1] =get_float_after_id(str, "length_b");
        }else if(!strcmp_case(tmp,"_cell_length_c")){
            cell[2]=get_float_after_id(str, "length_c");
        }else if(!strcmp_case(tmp,"_cell_angle_alpha")){
            cell[3]=get_float_after_id(str, "angle_alpha");
        }else if(!strcmp_case(tmp,"_cell_angle_beta")){
            cell[4]=get_float_after_id(str, "angle_beta");
        }else if(!strcmp_case(tmp,"_cell_angle_gamma")){
            cell[5]=get_float_after_id(str, "angle_gamma");
        }

        rid_of_front_end_space(str);
        nt++;
        
        if(strlen(str)<1 || str[0]=='#') continue;
        if(nline>10 && !strncmp(str,"data_", 5))break; //next block
        lines[nline] = new char [strlen(str)+1];
        strcpy(lines[nline],str);
        line_index[nline] = nt;
        nline++;
    }
//    printf("nline, nline_tmp= %s %d %d\n", inpfile, nline,nline_tmp);

    cryst_infor(fout, cell,(char *)"?" , 0);
    

    nitem=0;
    for(i=0; i<nline-5; i++){  //get cif items
        if(!strncmp(lines[i+0],"_refln_",7) &&
           !strncmp(lines[i+1],"_refln_",7) &&
           !strncmp(lines[i+2],"_refln_",7) &&
           !strncmp(lines[i+3],"_refln_",7)){
            
            nitem=0;
            item = new char *[80];
            for(j=i; j<nline-5 ; j++){
                if(strncmp(lines[j], "_refln_", 7)) {
                    start_hkl=j;
                    break;
                }
                item[nitem] = new char [strlen(lines[j])+1];
                sscanf(lines[j], "%s", item[nitem]);
                nitem++;
            }
            if(strncmp(lines[j], "_refln_", 7)) break; 
        }
    }
    
    if(nitem<=0) {
        printf("The input file (%s) has no correct tokens. \n", inpfile);
        printf("Please make sure it is a CIF file for small molecule.\n");
        if(lines && nline>0) free_memory_2d(lines, nline);
        fclose(finp);
        return;
    }
    
    get_cif_item_index(nitem, item, index, &nitem_tot);
    
    n=0;
    for(i=start_hkl; i<nline; i++){
        strcpy(tmp,lines[i]);
        line = string_token(tmp, " " , &nstr);
        
        if(nstr != nitem ){
            printf("Error at line = %d\n", line_index[i]);
            continue;
        }
        n++;
        write_new_mmcif(fout, n, item, index, line, nitem_tot);
        if(nstr>0) free_memory_2d(line, nstr);
    }
    if(item && nitem) free_memory_2d(item, nitem);
    if(lines && nline) free_memory_2d(lines, nline);
    delete [] line_index;
    
    
    fclose(finp);
}

void get_cif_item_index(int n, char **item, int index[], int *nitem_tot)  
//correspondance of total number of CIF and mmCIF items.
{
    int i;
    
    for(i=0; i<n; i++){
        if(!strcmp_case(item[i], "_refln_index_h")){
            strcpy(item[i],  "_refln.index_h");
            index[0]=i;
        }else if(!strcmp_case(item[i], "_refln_index_k")){
            strcpy(item[i],  "_refln.index_k");
            index[1]=i;
        }else if(!strcmp_case(item[i], "_refln_index_l")){
            strcpy(item[i],  "_refln.index_l");
            index[2]=i;
        }else if(!strcmp_case(item[i], "_refln_F_meas")){
            strcpy(item[i],  "_refln.F_meas_au");
            index[3]=i;
        }else if(!strcmp_case(item[i], "_refln_F_sigma")){
            strcpy(item[i],  "_refln.F_meas_sigma_au");
            index[4]=i;
        }else if(!strcmp_case(item[i], "_refln_F_squared_meas")){
            strcpy(item[i],  "_refln.intensity_meas");
            
            index[5]=i;
        }else if(!strcmp_case(item[i], "_refln_F_squared_sigma")){
            strcpy(item[i],  "_refln.intensity_sigma");
            index[6]=i;

        }else if(!strcmp_case(item[i], "_refln_F_calc")){
            strcpy(item[i],  "_refln.F_calc_au");
            index[7]=i;
        }else if(!strcmp_case(item[i], "_refln_F_squared_calc")){
            strcpy(item[i],  "_refln.F_squared_calc");
            index[8]=i;
            
        }else if(!strcmp_case(item[i], "_refln_observed_status")){
            strcpy(item[i],  "_refln.status");
            index[9]=i;

        }else if(!strcmp_case(item[i], "_refln_intensity_meas")){
            strcpy(item[i],  "_refln.intensity_meas");
            index[10]=i;
        }else if(!strcmp_case(item[i], "_refln_intensity_sigma")){
            strcpy(item[i],  "_refln.intensity_sigma");
            index[11]=i;
            
        }else if(!strcmp_case(item[i], "_refln_fom")){
            strcpy(item[i],  "_refln.fom");
            index[12]=i;
            
        }else if(!strcmp_case(item[i], "_refln_phase_calc")){
            strcpy(item[i],  "_refln.phase_calc");
            index[13]=i;
            
        }else if(!strcmp_case(item[i], "_refln_status")){
            strcpy(item[i],  "_refln.status");
            index[14]=i;
        }
        
    }
    *nitem_tot=15;
}

void write_new_mmcif(FILE *fout, int n, char **item, int *index, char **line,
                     int nitem_tot)
{
    int i, k=-1;
    
    if(n==1){ // write the possible items
        fprintf(fout,"\n#\nloop_\n");
        fprintf(fout, "_refln.crystal_id \n");
        fprintf(fout, "_refln.wavelength_id \n");
        fprintf(fout, "_refln.scale_group_code\n");
        
        for(i=0; i<nitem_tot; i++){
            if(index[i] >=0){
                fprintf(fout,"%s\n", item[index[i]]);
            }
            
        }
    }

    for(i=0; i<nitem_tot; i++){
        if(index[i] >=0){
            if(strstr(item[index[i]], "refln.status")){
//                printf("index for status=%d\n", i);
                k=i;
            }
        }
    }

    
    for(i=0; i<nitem_tot; i++){// write the possible values
        if(i<=2 && i>=0){
            if(i==0)
                fprintf(fout,"1 1 1 ");
            if(index[i] >=0)
                fprintf(fout," %4s", line[index[i]]);
        }else if(i==k){ //for status
            if(atof(line[index[i]])>0.5)
                fprintf(fout," f ");               
            else
                fprintf(fout," o ");               
                
        }else{
            
            if(index[i] >=0)
                fprintf(fout," %10s", line[index[i]]);
        }
                
//        if(index[i] >=0)fprintf(fout," %s", line[index[i]]);
    }
    
    fprintf(fout,"\n");
    
}

 
void sf_data_header(FILE *fout, int I_F_type, int status )
// status>0 Rfree exist
// I_F_type=0, F & I ; =1, F only; =2, I only; =3, I(+)(-) only; =4, F(+)(-) only;
{
    fprintf(fout, "#\n");

    fprintf(fout, "loop_\n");
    fprintf(fout, "_refln.crystal_id \n");
    fprintf(fout, "_refln.wavelength_id \n");
    fprintf(fout, "_refln.scale_group_code\n");
    fprintf(fout, "_refln.index_h\n");
    fprintf(fout, "_refln.index_k\n");
    fprintf(fout, "_refln.index_l\n");
    
    
    if(I_F_type ==0 ){
        fprintf(fout, "_refln.F_meas_au\n" );
        fprintf(fout, "_refln.F_meas_sigma_au \n" );
        fprintf(fout, "_refln.intensity_meas\n" );
        fprintf(fout, "_refln.intensity_sigma\n" );
        
    }else if(I_F_type ==1){  
        fprintf(fout, "_refln.F_meas_au\n" );
        fprintf(fout, "_refln.F_meas_sigma_au \n" );
        
    }else if(I_F_type ==2){  
        fprintf(fout, "_refln.intensity_meas\n" );
        fprintf(fout, "_refln.intensity_sigma\n" );
        
    }else if(I_F_type ==3){  
        fprintf(fout, "_refln.pdbx_I_plus\n" );
        fprintf(fout, "_refln.pdbx_I_plus_sigma\n" );
        fprintf(fout, "_refln.pdbx_I_minus\n" );
        fprintf(fout, "_refln.pdbx_I_minus_sigma\n" );
        
    }else if(I_F_type ==4){  
        fprintf(fout, "_refln.pdbx_F_plus\n" );
        fprintf(fout, "_refln.pdbx_F_plus_sigma\n" );
        fprintf(fout, "_refln.pdbx_F_minus\n" );
        fprintf(fout, "_refln.pdbx_F_minus_sigma\n" );
        
    } else{
        printf("Data-type is wrong! (It should be either (F) or (I))\n");
        exit(0);
    }
    if(status >0 )fprintf(fout, "_refln.status\n" );

}



/////////////////////////// Unix convert MTZ to mmcif /////////////////////
/*
 mmCIF                            Data_type  MTZ labels   
                                              (possible)   
 _refln.wavelength_id                 .           .              Always written
 _refln.crystal_id                    .           .              Always written
 _refln.scale_group_code              .           .              Always written
 _refln.index_h                       H           K              Always written
 _refln.index_k                       H           K              Always written
 _refln.index_l                       H           L              Always written
 _refln.status            	      I           RFREE          Always written
                                                                                           
 _refln.F_meas_au         	      F           FP, F_X2    
 _refln.F_meas_sigma_au               Q           SIGFP, SIGF_X2  
 _refln.F_calc            	      F           FC     
 _refln.phase_calc        	      P           PHIC   
 _refln.phase_meas        	      P           PHIB   
 _refln.fom               	      W           FOM    
 _refln.intensity_meas    	      J           I      
 _refln.intensity_sigma   	      Q           SIGI   
                                                                                           
 _refln.pdbx_HL_A_iso                 A       	HLA    
 _refln.pdbx_HL_B_iso                 A       	HLB    
 _refln.pdbx_HL_C_iso                 A       	HLC    
 _refln.pdbx_HL_D_iso                 A       	HLD    
 _refln.pdbx_F_plus         	      G           F(+)   
 _refln.pdbx_F_plus_sigma   	      L           SIGF(+)
 _refln.pdbx_F_minus        	      G           F(-)   
 _refln.pdbx_F_minus_sigma  	      L           SIGF(-)
 _refln.pdbx_anom_difference          D           DANO   
 _refln.pdbx_anom_difference_sigma    Q           SIGDANO
 _refln.pdbx_I_plus                   K           I(+)   
 _refln.pdbx_I_plus_sigma             M           SIGI(+)
 _refln.pdbx_I_minus                  K           I(-)   
 _refln.pdbx_I_minus_sigma            M           SIGI(-)      

*/

 
                         
void mtz2mmcif_multi(char *inpfile, char *outfile)
// get multiple data set.
{
    char **label_ptr=NULL, **type_ptr=NULL, **sets_ptr=NULL;
    char outfile1[256], outfile2[256],command[512];
    char label[2000], type[256], sets[256];
    int nlabel=0, ntype=0,nsets=0, i,j,min, max;
    
    get_type_label_sets(inpfile, label, type, sets);
    
    label_ptr = string_token(label," ", &nlabel);
    type_ptr = string_token(type," ", &ntype);
    sets_ptr = string_token(sets," ", &nsets);

    if(nlabel != ntype || nlabel != nsets){
        printf("Error! There is a problem with data set.\n");
        return;
    }
    min=90;
    max=-1;
    
    printf("items  set  type  label\n");
    for(i=0; i<nsets; i++){
        printf("   %2d   %s    %s     %s\n",
               i+1,sets_ptr[i], type_ptr[i], label_ptr[i]);
    }
    
    for(i=3; i<nsets; i++){
        j=atoi(sets_ptr[i]);
        if(j<min)min=j;
        if(j>max)max=j;
    }

    NFIRST=first_set_num(label_ptr, type_ptr, sets_ptr, ntype);
    
    printf("Total Columns =%d, min=%d, max=%d, data_set=%d\n",ntype, min,max, NFIRST);

   
    if(max==NFIRST){//only one set
        mtz2mmcif(inpfile,outfile, label_ptr, type_ptr, sets_ptr, ntype, 1);  
    }else{
        
        sprintf(outfile1, "%s_1", outfile);
        sprintf(outfile2, "%s_2", outfile);
        delete_file(outfile1);
        delete_file(outfile2);
        
        mtz2mmcif(inpfile,outfile1, label_ptr, type_ptr, sets_ptr, ntype, 1);
        mtz2mmcif_phs(inpfile,outfile2, label_ptr, type_ptr, sets_ptr, ntype,2);
        
        sprintf(command, "/bin/csh -c \" /bin/cat  %s %s  > %s\" ",outfile1,outfile2,outfile);
        system(command);

        delete_file(outfile1);
        delete_file(outfile2);
    }
}

void get_type_label_sets(char *inpfile, char *label,char *type,  char *sets)
//get data sets, labels, types
{
    char  str[2000], command[512], log[256];
//    int pid;
    
    FILE *flog=NULL;
    
//    pid = getpid();
    sprintf(log, "mtzdmp.log"); //get log file only
   
    sprintf(command, "/bin/csh -c \"source %s/include/ccp4.setup;%s/etc/mtzdmp  %s  > /dev/null \"", CCP4,CCP4, inpfile);
    system(command);
    
    sprintf(command, "/bin/csh -c \"source %s/include/ccp4.setup;%s/etc/mtzdmp  %s  >%s \"", CCP4,CCP4, inpfile, log);
    system(command);

    
    if((flog=fopen(log, "r"))==NULL){
        printf("Error, can not open the LOGfile (%s)\n",log);
        exit(0);
    }

    while(fgets(str, sizeof str, flog)!=NULL){ 
      if ( strstr(str,"* Associated datasets :")){
            while(fgets(str, sizeof str, flog)!=NULL){
                
                rid_of_front_end_space(str);
                if(strlen(str)>2){
                    strcpy(sets,str);
                    break;
                }
            }
        }else if ( strstr(str,"* Column Labels :")){
            while(fgets(str, sizeof str, flog)!=NULL){
                
                rid_of_front_end_space(str);
                if(strlen(str)>2){
                    strcpy(label,str);
                    break;
                }
            }
        }else if( strstr(str,"* Column Types :")){
            while(fgets(str, sizeof str, flog)!=NULL){
                rid_of_front_end_space(str);
                if(strlen(str)>2){
                    strcpy(type,str);
                    break;
                }
            }
        }        
    }
    fclose(flog);
//    delete_file(log);
    
}

int first_set_num(char **label_ptr,char **type_ptr,char **sets_ptr,int ntype)
// only test F and I for the data set (return the data set number).
{
    int j, n=0;
    
    for(j=3; j<ntype-1; j++){
//        printf("Data items=%d %s  %s \n", j, type_ptr[j], label_ptr[j]);
        
        if      (type_ptr[j][0] == 'F' && type_ptr[j+1][0] == 'Q' && toupper(label_ptr[j][0])== 'F' ){
            n=atoi(sets_ptr[j]);
            break;
        }else if(type_ptr[j][0] == 'J' && type_ptr[j+1][0] == 'Q' && toupper(label_ptr[j][0])== 'I' ) {
            n=atoi(sets_ptr[j]);
            break;
        }else if(type_ptr[j][0] == 'L' && type_ptr[j+1][0] == 'G' && toupper(label_ptr[j][0])== 'F' ) {
            n=atoi(sets_ptr[j]);
            break;
        }else if(type_ptr[j][0] == 'K' && type_ptr[j+1][0] == 'M' && toupper(label_ptr[j][0])== 'I' ) {
            n=atoi(sets_ptr[j]);
            break;
        }
    }
    return n;
}



void mtz2mmcif(char *filename, char *outfile, char **label_ptr,
               char **type_ptr,char **sets_ptr,int ntype, int set_num) 
// using mtz2various
{
    
    char script[512], command[256] ;
    FILE *fp=NULL;

    
    sprintf(script, "%smtz2cif_conv_%d.scr",PATHTMP, set_num);
    delete_file(script);
    
    if((fp=fopen(script, "w"))==NULL){
        printf("Error, no written permission in directory (%s)!\n", PATHTMP);
        exit(0);
    }
    
    fprintf(fp,"#!/bin/csh \n\n");
    fprintf(fp,"# If you want the log file, replace /dev/null to test.log\n");
    
    fprintf(fp,"%s/bin/mtz2various HKLIN %s HKLOUT %s << EOF> /dev/null\n",
            CCP4, filename,outfile);
    fprintf(fp,"OUTPUT CIF data_mtz2cif \n");
    
    if(set_num==1){
        print_labin_1(fp,label_ptr,type_ptr,sets_ptr,ntype);
    }else{
        print_labin_2(fp,label_ptr,type_ptr,sets_ptr,ntype);
    }
    
    fprintf(fp,"END\n");
    fprintf(fp,"EOF\n");
    fprintf(fp,"#\n");

    fclose(fp);
    
    sprintf(command, "/bin/csh -c \"chmod +x %s\"", script);
    system(command);
    sprintf(command, "/bin/csh -c \"source %s/include/ccp4.setup; %s\" >/dev/null", CCP4,script);
    system(command);
    
//    delete_file(script);

}


void print_labin_1(FILE *finp, char **label_ptr,char **type_ptr,
                   char **sets_ptr,int ntype)
//for first data set
{
    
    int  j,  d_set;
   

    for(j=3; j<ntype-1; j++){
        if(type_ptr[j][0] =='G'  && type_ptr[j+1][0] == 'L'){
//            F_minus=1;
            break;
        }
    }
    d_set = NFIRST;
    
    
    fprintf(finp,"LABIN  ");  

    f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, //FP, sigma
               'F','Q','F', d_set, "FP", "SIGFP",finp);
    
    f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, //intensity, sigma
               'J','Q','I', d_set, "I", "SIGI",finp);
    
    f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, //Ano_diff, sigma
               'D','Q', 'D', d_set, "DP", "SIGDP",finp);
    
//    if(F_minus>0){
        
    f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, // F(+), sigma 
               'G', 'L',  'F',  d_set, "F(+)", "SIGF(+)",finp);
    
    f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, // F(-), sigma 
               'G', 'L',  'F',  d_set, "F(-)", "SIGF(-)",finp);


//    }else{
              
    f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, // I(+),sigma  
               'K', 'M',  'I',  d_set, "I(+)", "SIGI(+)",finp);
        
    f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, // I(-),sigma  
               'K', 'M',  'I',  d_set, "I(-)", "SIGI(-)",finp);

//    }
    

    for(j=3; j<ntype-1; j++){
        if(type_ptr[j][0] == 'F' && type_ptr[j+1][0] != 'Q'){ //FC
            if(!strcmp_case(label_ptr[j], "FC") || !strcmp_case(label_ptr[j], "Fcal")){
                fprintf(finp,"FC=%s  ",label_ptr[j] );
                break;  
            }
        }
    }
    
    for(j=3; j<ntype; j++){
//        printf("set=%s : type=%s : label=%s \n",sets_ptr[j],type_ptr[j], label_ptr[j]);
        
        if(type_ptr[j][0] == 'P' ){ //PHIC, PHIB
            if(strstr_case(label_ptr[j], "PHIC") || strstr_case(label_ptr[j], "AC")){
                        
                fprintf(finp,"PHIC=%s  ",label_ptr[j] );
                
            }else if (strstr_case(label_ptr[j], "PHIB") ||
                      strstr_case(label_ptr[j], "PHIM")){
                fprintf(finp,"PHIB=%s  ",label_ptr[j] );
            }
        }else if(type_ptr[j][0] == 'A' ){ // HLA,HLB,HLC,HLD
            if(strstr_case(label_ptr[j], "HLA") ){
                fprintf(finp,"HLA=%s  ",label_ptr[j] );
            }else if(strstr_case(label_ptr[j], "HLB") ){
                fprintf(finp,"HLB=%s  ",label_ptr[j] );
            }else if(strstr_case(label_ptr[j], "HLC") ){
                fprintf(finp,"HLC=%s  ",label_ptr[j] );
            }else if(strstr_case(label_ptr[j], "HLD") ){
                fprintf(finp,"HLD=%s  ",label_ptr[j] );
            }
                    
        }else if(type_ptr[j][0] == 'W' ){ //FOM
            if(strstr_case(label_ptr[j], "FOM") ){
                fprintf(finp,"FOM=%s  ",label_ptr[j] );
            }
        }else if(type_ptr[j][0] == 'I' ){ //status
            if(strstr_case(label_ptr[j], "free") ||
               strstr_case(label_ptr[j], "flag") ||
               strstr_case(label_ptr[j], "TEST")){
                        
                fprintf(finp,"FREE=%s ",label_ptr[j] );
            }

        }
    }
    fprintf(finp,"\n");
}

void print_labin_2(FILE *finp, char **label_ptr,char **type_ptr,
                   char **sets_ptr,int ntype)
//data after the first set.
{
    int i, max=0, d_set;
    
    max=atoi(sets_ptr[ntype-1]);
    
    for(i=NFIRST+1; i<=max; i++){ // for more data sets
        d_set=i;
        
        fprintf(finp,"LABIN  ");  

        f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, //FP, sigma
                   'F','Q','F', d_set, "FP", "SIGFP",finp);
    
        f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, //intensity, sigma
                   'J','Q','I', d_set, "I", "SIGI",finp);
    
        f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, //Ano_diff, sigma
                   'D','Q', 'D', d_set, "DP", "SIGDP",finp);
    
//        if(F_minus>0){
        
        f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, // F(+), sigma 
                   'G', 'L',  'F',  d_set, "F(+)", "SIGF(+)",finp);
    
        f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, // F(-), sigma 
                   'G', 'L',  'F',  d_set, "F(-)", "SIGF(-)",finp);


//        }else{
              
        f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, // I(+),sigma  
                   'K', 'M',  'I',  d_set, "I(+)", "SIGI(+)",finp);
            
        f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, // I(-),sigma  
                   'K', 'M',  'I',  d_set, "I(-)", "SIGI(-)",finp);
//        }
        fprintf(finp,"\n");
    }
}

void f_sig_pair(char **label_ptr, char **type_ptr,char **sets_ptr, int ntype,
                const char type1, const char type2, const char lab1, int nset,
                const  char *asign_lab1, const char *asign_lab2, FILE *finp)
// get data pairs (F SIGF, I SIGI, ...)
{
    int j;

    for(j=3; j<ntype-1; j++){
        
        if(atoi(sets_ptr[j])!= nset) continue; //only for this set
        
        if(strstr(asign_lab1, "(+)")){
            
            if(type_ptr[j][0] ==type1  && type_ptr[j+1][0] == type2){ 
                if(strstr(label_ptr[j], "(+)") && strstr_case(label_ptr[j+1], "SIG")) {
                        
                    fprintf(finp,"%s=%s  ",asign_lab1, label_ptr[j] );
                    fprintf(finp,"%s=%s  ",asign_lab2, label_ptr[j+1] );
                    break;
                }
            }
        }else if(strstr(asign_lab1, "(-)")){
            
            if(type_ptr[j][0] ==type1  && type_ptr[j+1][0] == type2){ 
                if(strstr(label_ptr[j], "(-)") && strstr_case(label_ptr[j+1], "SIG")) {
                        
                    fprintf(finp,"%s=%s  ",asign_lab1, label_ptr[j] );
                    fprintf(finp,"%s=%s  ",asign_lab2, label_ptr[j+1] );
                    break;
                }
            }
        }else{
              
            if(type_ptr[j][0] == type1 && toupper(label_ptr[j][0])== lab1 &&
               type_ptr[j+1][0] == type2 && strstr_case(label_ptr[j+1], "SIG")){ 
                        
                fprintf(finp,"%s=%s  ",asign_lab1, label_ptr[j] );
                fprintf(finp,"%s=%s  ",asign_lab2, label_ptr[j+1] );
                break;
                
            }else if(j<ntype-2 &&
                     type_ptr[j][0] == type1 && toupper(label_ptr[j][0])== lab1 &&
                     type_ptr[j+2][0] == type2 && strstr_case(label_ptr[j+2], "SIG")){ 
                     
                fprintf(finp,"%s=%s  ",asign_lab1, label_ptr[j] );
                fprintf(finp,"%s=%s  ",asign_lab2, label_ptr[j+2] );
                break;
                
            }else if(type_ptr[j][0] == type1 && toupper(label_ptr[j][0])== lab1 &&
                     type_ptr[j-1][0] == type2 && strstr_case(label_ptr[j-1], "SIG")){ 
                     
                fprintf(finp,"%s=%s  ",asign_lab1, label_ptr[j] );
                fprintf(finp,"%s=%s  ",asign_lab2, label_ptr[j-1] );
                break;
            }else if(type_ptr[j][0] == type1 && toupper(label_ptr[j][0])== lab1){ 
                     
                fprintf(finp,"%s=%s  ",asign_lab1, label_ptr[j] );
                break;
            }
            
        }
        
    }
}

void mtz2mmcif_phs( char *filename, char *outfile, char **label_ptr,
               char **type_ptr, char **sets_ptr, int ntype, int set_num)
// using Peter's script to convert multi-data of MTZ to a single mmCIF file
        
{
    char script[512], command[256] ;
    FILE *fp=NULL;

    if(strlen(outfile)<=0)  sprintf(outfile, "%s.cif", filename);
    
    sprintf(script, "%smtz2cif_mult_%d.scr",PATHTMP, set_num);
    delete_file(script);
    if((fp=fopen(script, "w"))==NULL){
        printf("Error, no written permission in directory (%s)!\n", PATHTMP);
        exit(0);
    }
    
    fprintf(fp,"#!/bin/csh \n");
    fprintf(fp,"#Script to run the MTZ2CIF program\n");
    fprintf(fp,"#\n");
    fprintf(fp,"#NB MODE keyword allows the user to specify whether to output\n");
    fprintf(fp,"#the standard CCP4 CIF tokens or the RCSB tokens\n\n");
    
    fprintf(fp,"%s  HKLIN %s  \\\nHKLOUT %s << EOF  \n", MTZ2CIF, filename,outfile);
    
    print_labin_2(fp,label_ptr,type_ptr,sets_ptr,ntype);
   
    fprintf(fp,"datablock data_PHASE\n");
    fprintf(fp,"mode PDBX\n");
    fprintf(fp,"end\n");
    fprintf(fp,"EOF\n");
    fprintf(fp,"#\n");

    fclose(fp);
    
    sprintf(command, "/bin/csh -c \"chmod +x %s\"", script);
    system(command);
    sprintf(command, "/bin/csh -c \"source %s/include/ccp4.setup; %s \"  >/dev/null", CCP4,script);
    system(command);
    
//    delete_file(script);
}


